/**
*
* $Id: Titov V.cpp
* (c) 2005 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* Autopilot by rjcroy
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/

#include "Hotol.h"


// 1. vertical lift component (wings and body)
// Borrowed from Mike Norman's Shuttle code

void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*15.0;
	static const double istep = 1.0/step;
	static const int nabsc = 25;
	static const double CL[nabsc] = {0.1, 0.17, 0.2, 0.2, 0.17, 0.1, 0, -0.11, -0.24, -0.38, -0.5, -0.5, -0.02, 0.6355, 0.63, 0.46, 0.28, 0.13, 0.0, -0.16, -0.26, -0.29, -0.24, -0.1, 0.1};
	static const double CM[nabsc] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0.002, 0.004, 0.0025,0.0012, 0,-0.0012,-0.0007, 0, 0, 0, 0, 0, 0, 0, 0, 0};
	// lift and moment coefficients from -180 to 180 in 15 degree steps.
	// This uses a documented lift slope of 0.0437/deg, everything else is rather ad-hoc

	// change for drag increase at high Mach:
	// Subsonic Equivalents

	double Cl0, Cl1, Cd0, Cd1, Cd2;
	if(M>=5.0) {
	Cl0 = -0.207040;
	Cl1 = 1.675600;
	Cd0 = 0.058540;
	Cd1 = -0.05767;
	Cd2 = 1.3632;

	*cl = Cl0 + Cl1*aoa;
	*cd = Cd0 + Cd1*aoa + Cd2*aoa* aoa;
	}
	aoa += PI;
	int idx = max (0, min (23, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cm = (CM[idx] + (CM[idx+1]-CM[idx])*d);
		if(M<1.25) { 
			*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
			*cd = 0.03 + oapiGetInducedDrag (*cl, 2.266, 0.75);
		}
	else if(M<5.0) { 
		*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
		*cd = 0.07 + oapiGetInducedDrag (*cl, 2.266, 0.75);
		}
}


/*
void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	const int nabsc = 9;
	static const double AOA[nabsc] = {-180*RAD,-60*RAD,-30*RAD, 0*RAD, 15*RAD,25*RAD,30*RAD,60*RAD,180*RAD};
	static const double CL[nabsc]  = {       0,      0,   -0.4,     0,    0.7,     1,   0.8,     0,      0};
	static const double CM[nabsc]  = {       0,      0,  0.014, 0.004,-0.0074,-0.094,-0.012,     0,      0};
	for (int i = 0; i < nabsc-1 && AOA[i+1] < aoa; i++);
	double f = (aoa-AOA[i]) / (AOA[i+1]-AOA[i]);
	*cl = CL[i] + (CL[i+1]-CL[i]) * f;  // aoa-dependent lift coefficient
	*cm = CM[i] + (CM[i+1]-CM[i]) * f;  // aoa-dependent moment coefficient
	double saoa = sin(aoa);
	double pd = 0.015 + 0.4*saoa*saoa;  // profile drag
	*cd = pd + oapiGetInducedDrag (*cl, 1.5, 0.7) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
	// profile drag + (lift-)induced drag + transonic/supersonic wave (compressibility) drag
	if (M > 1.0) *cd *= 2.2;
	else *cd *= 0.6;
}
*/

// 2. horizontal lift component (vertical stabilisers and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 8;
	static const double BETA[nabsc] = {-180*RAD,-135*RAD,-90*RAD,-45*RAD,45*RAD,90*RAD,135*RAD,180*RAD};
	static const double CL[nabsc]   = {       0,    +0.3,      0,   -0.3,  +0.3,     0,   -0.3,      0};
	for (i = 0; i < nabsc-1 && BETA[i+1] < beta; i++);
	*cl = CL[i] + (CL[i+1]-CL[i]) * (beta-BETA[i]) / (BETA[i+1]-BETA[i]);
	*cm = 0.0;
	*cd = 0.015 + oapiGetInducedDrag (*cl, 1.5, 0.6) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
	
	if (M > 1.0) *cd *= 2.2;
	else *cd *= 0.6;
}


																					
void HOTOL::InitialiseGuidance()
{
	

//	time=0.0;

	pitchinput = 0.0;
	rollinput = 0.0;
	engineignited = false;
	last_simt=0.0;
	ignition_time=0.0;
	first_guidance_pass = true;

	// rjcroy Guidance variables

	INCLINATION = 51.63 ;		// degrees

	guidance = false;			// guidance switch
	Alt_Flag = false;			// True if max launch altitude reached
	MECO_Cond = false;			// True if MECO conditions met
	First_Guidance_Pass = false;
	T_minus_10 = true;			// True at T minus 10 seconds.
	TZero = -100;				// Time Zero - Launch time
	PC_D = 0.9;					// Pitch control coefficient D for d_theta /d_t
	PC_K = 0.2;					//    "    "         "       K  "  theta
	YC_D = 0.5	;				// Yaw Control
	YC_K = 0.3;
	PC_D_FINAL = 0.9;			// Pitch control coefficients for final section.
	PC_K_FINAL = 0.6; 
	D_FINAL  = 0.8;				// Vertical velocity control coefficients
	K_FINAL = 0.8;
	ENG_THRST_2 = 0 ;		// Thrust rating of Stage 2 engine. (N)  
	i_last = 0;					// value of incliniation in previous iteration.
	Planet_Name = new char[6];
	strcpy( Planet_Name, "Earth");
	R_EARTH = 6371000;			// radius of the earth (m)
	U_EARTH = 3.986e5 * 1000 * 1000 * 1000; //( m^3/sec^2)
	Ob_EARTH = 23.45 * RAD ;	// Obliquity of the Earth (rad)
	APO_ALT = 300 ;				// Altitude of apoapsis
//	APO_ALT = 1705;

	//  inclination correction factor constant. (3580)
	ICF_K = 62.008 * INCLINATION*INCLINATION 
			- 6848.7*INCLINATION + 189807;	

	DisplayIncSP = false;		// toggle for displaying the inclination setpoint.

	NewIncSP = false ;			// True if a user has entered a new inclination SP
	PostMECO = false ;			// switch for cleaning up after MECO

	BadHeading = false;			// true if soyuz is at an azimuth not suitable for the G&CS



}

double HOTOL::GetCPitch(double t)
{
	int i;
	if (t>met[N-1]) return cpitch[N-1];
	i=1;
	while (met[i]<t) i++;
	return cpitch[i-1]+(cpitch[i]-cpitch[i-1])/(met[i]-met[i-1])*(t-met[i-1]);
}

// rjcroy Lateral Guidance System ( called by Guidance_Routine() ).

void HOTOL::Lateral_Guidance(VESSEL *vessel, double SimT, double SimT_Last)
{
	// Lateral Guidance System 
/*
		double hdg, diff, rate, amount, targetrate, TgtHdg;

		if (MET > 35.0) {
			targetrate = ((GetBank()*180/PI)-180)/-1.0;
			if (GetBank()<0) targetrate = -1.0*targetrate;

			if (MET > 55.0) {

				oapiGetHeading(GetHandle(),&hdg);
//				hdg = vst.arot.y * 180/PI;
				hdg *= 180/PI;
//				hdg += 180;

				GetRelativeVel (hEarth, Rel_Vel);
				GetEquPos( Longitude, Latitude, Radius );
				Velocity = sqrt((Rel_Vel.x*Rel_Vel.x) + (Rel_Vel.y*Rel_Vel.y) + (Rel_Vel.z*Rel_Vel.z));

				oapiGetRelativeVel(hVessel, hEarth, &Rel_Vel);
				oapiGetRelativePos(hVessel, hEarth, &Rel_Pos);
				
				// Change ecliptic rel. vectors to equatorial vectors.
 				temp.x = Rel_Vel.x;
				temp.y = Rel_Vel.z;
				temp.z = Rel_Vel.y;
				veccpy( Rel_Vel, temp);
				temp.x = Rel_Pos.x;
				temp.y = Rel_Pos.z;
				temp.z = Rel_Pos.y;
				veccpy( Rel_Pos, temp);
				temp.x = Rel_Pos.x ;
				temp.y = cos(-Ob_EARTH)* Rel_Pos.y + sin(-Ob_EARTH)*Rel_Pos.z;
				temp.z =  -sin(-Ob_EARTH)* Rel_Pos.y + cos(-Ob_EARTH)*Rel_Pos.z;
				veccpy( Rel_Pos, temp);
				temp.x = Rel_Vel.x ;
				temp.y = cos(-Ob_EARTH)* Rel_Vel.y + sin(-Ob_EARTH)*Rel_Vel.z;
				temp.z =  -sin(-Ob_EARTH)* Rel_Vel.y + cos(-Ob_EARTH)*Rel_Vel.z;
				veccpy( Rel_Vel, temp);

				h_vector = crossp(Rel_Vel, Rel_Pos);
				h_abs = sqrt((h_vector.x*h_vector.x) + (h_vector.y*h_vector.y) + (h_vector.z*h_vector.z));
				i = 180 - acos(h_vector.z / h_abs) * DEG;
				i_dot = (i - i_last) / (MET - SimT_Last) ;
				i_last = i;
				ICF = Velocity / ICF_K;

				// Calculated error in inclination as output signal
				i_err = (i - INCLINATION) * (150 * ICF) + 250 * ICF * i_dot;
				if (i_err > ICF )
				{	
					i_err = ICF;
				}
				else if ( i_err < -ICF) 
				{
					i_err = -ICF;
				}
							
				double Phrad;
					//= (PadHeading-180)*PI/180;

				if ((Rel_Vel.z < 110.0) || (bgoingsouth)) {
					Phrad = PI/2 + (PI/2 - asin(cos(INCLINATION*PI/180)/cos(Latitude)));
					if ((PadHeading-180) > 180)
						Phrad = 3*PI/2 + (3*PI/2 - asin(cos(INCLINATION*PI/180)/cos(Latitude)));
					TgtHdg = Phrad*DEG;
					TgtHdg -= i_err;
					bgoingsouth = true;
				}
				else {
					Phrad =  asin(cos(INCLINATION*PI/180)/cos(Latitude));
					TgtHdg = Phrad*DEG;
					TgtHdg += i_err;
				}

				if ((TgtHdg > 90.0) || (PadHeading-180 >= 90.0))
					bgoingsouth = true;

				if ((Latitude*DEG >= i*0.98) && (fabs(i-INCLINATION)<0.02))
					bgoingsouth = true;

//				sprintf(oapiDebugString(),"Latitude %.2f  inc %.2f TGTINC %.2f going %1d",Latitude*DEG, i, INCLINATION, bgoingsouth);
//				sprintf(oapiDebugString(),"TgtHdg %.2f  inc %.2f TGTINC %.2f Phrad %.2f PadHdg %.2f tmpy %.2f",TgtHdg, i, INCLINATION, Phrad*DEG,PadHeading-180,Rel_Vel.z);

				diff = hdg - (TgtHdg);
//				if (ROLL < 90.0) diff = hdg - TgtHdg + 180 - ROLL;
//				diff = hdg - (PadHeading - 180);
				if( diff > 180.0 )
					diff -= 360.0;
				else if( diff < -180.0 )
					diff += 360.0;
				targetrate = diff/+1.0;
			}

			if( targetrate > 8.0 )
				targetrate = 8.0;
			else if( targetrate < -8.0 )
				targetrate = -8.0;


			rate = vst.vrot.y * 180/PI;
			amount = fabs (targetrate/8.0);
			if (MET > 0) amount = amount / (MET/10);

//		sprintf(oapiDebugString(),"Bank %.1f Heading %.1f TgtHdg %.1f diff %.1f Targetrate %.1f",GetBank()*180/PI,hdg,TgtHdg,diff, targetrate);

			if( fabs( rate-targetrate ) > 0.05 ) {
				if( rate < targetrate ) {
//					IncThrusterGroupLevel( THGROUP_ATT_YAWRIGHT, 0.05 );
//					SetThrusterGroupLevel( THGROUP_ATT_YAWLEFT, 0.0 );
					yawinput = -1*amount;
				}
				else {
//					IncThrusterGroupLevel( THGROUP_ATT_YAWLEFT, 0.05);
//					SetThrusterGroupLevel( THGROUP_ATT_YAWRIGHT, 0.0 );
					yawinput = amount;
				}
			}
			else {
//				SetThrusterGroupLevel( THGROUP_ATT_YAWLEFT, 0.0 );
//				SetThrusterGroupLevel( THGROUP_ATT_YAWRIGHT, 0.0 );
				yawinput = 0.0;
			}
		}
*/
}

// rjcroy Guidance & Control System
void HOTOL::Guidance_Routine(VESSEL *vessel, double SimT, double SimT_Last)
{

	hVessel = GetHandle();
	hEarth = oapiGetGbodyByName ( Planet_Name );


	if (T_minus_10)
	{
		// Record launch time
		if (TZero == 0){
			TZero = SimT + 1;
		}

		if (MET > -5 && MET < -3) {
//			oapiSetEngineLevel(hVessel, ENGINE_MAIN, 0.3);
//			LaunchClamps();
		}
		else if (MET > -3 && MET < 0) {
//			oapiSetEngineLevel(hVessel, ENGINE_MAIN, (0.2*MET + 1));
//			LaunchClamps();
			INCLINATION = 180 - acos(sin(PadHeading*PI/180)*cos(Latitude))*DEG;
		}
		else if (MET > 0)
		{
			T_minus_10 = false;
			First_Guidance_Pass = true;
			oapiSetEngineLevel(hVessel, ENGINE_MAIN, 1);

			GetStatus(vst);
	
			initialgamma = vst.arot.z * 180/PI;
			launchrotate =	PadHeading - initialheading - 180 + ROLL;

			if( launchrotate < -180.0 )
				launchrotate += 360.0;
			else if( launchrotate > 180.0 )
				launchrotate -= 360.0;

		}
		sprintf(oapiDebugString(),"T - %.1f", fabs(MET));
	}		

	else if (First_Guidance_Pass) 
	{
		
		// Start the main engines
		oapiSetEngineLevel(hVessel, ENGINE_MAIN, 1);

		// Unset the first pass flag
		First_Guidance_Pass = false;
	}

	else if (!Alt_Flag) 
	{
		if (oapiGetFocusObject() == GetHandle())
			sprintf(oapiDebugString(),"T + %.1f", fabs(MET));

		if (initialgamma == 0.0)
			initialgamma = vst.arot.z * 180/PI;

		// Get correct pitch from table
		Pitch_SP=GetCPitch(MET);
		MJD_Time = oapiGetSimMJD();
		GetElements( elements, MJD_Time );
		Alt_Apoapsis = (elements.a * ( 1 + elements.e )/1000) - 6371; 
		if (Alt_Apoapsis > 100.0) {
			Pitch_SP = Pitch_SP/4.0;			// Flydown -- we've met our altitude goal
			if (ROLL < 90.0)
				if ((GetAOA()*DEG > -1.0))
					Pitch_SP += 0+0.9*GetAOA()*DEG;
			else
				if ((GetAOA()*DEG < 1.0))
					Pitch_SP += 1-0.9*GetAOA()*DEG;
		}

//		sprintf(oapiDebugString(),"Alt_Apoapsis %.1f Pitch_SP %.1f CPitch %.1f",Alt_Apoapsis, Pitch_SP, GetCPitch(MET));

		// Control to pitch setpoint
		Pitch=GetPitch(  );
		Pitch = Pitch * DEG;
		Pitch_Error = Pitch - Pitch_SP;
		if (ROLL < 90.0)
			Pitch_Error = Pitch_SP - Pitch;

		GetAngularVel( Angular_Velocity ); 
		Pitch_Error_Dot = Angular_Velocity.x * DEG; // get pitch change rate in degrees/s

		Pitch_Cntrl_Sig =  - PC_D * Pitch_Error_Dot + PC_K * Pitch_Error;
//		if ((MET > 10.0) &&(GetBank() < PI/2) && (GetBank() > -PI/2))
//			Pitch_Cntrl_Sig *= -1.0;

		if (Pitch_Cntrl_Sig > 1 )
		{
//			SetAttitudeRotLevel( 0, 1); // 0 is y-axis
			pitchinput =-1.0;
		}
		else if (Pitch_Cntrl_Sig < -1) 
		{
//			SetAttitudeRotLevel( 0, -1);
			pitchinput = +1.0;
		}
		else {
//			SetAttitudeRotLevel( 0 , Pitch_Cntrl_Sig);
			pitchinput = -Pitch_Cntrl_Sig;
		}

	

		// Heading control
//		if (ROLL > 90.0) Lateral_Guidance(this, SimT, SimT_Last);


		// Roll control
		// Adapted from Hendo's Ariane V



		if( MET > 0.0) {
				
//		vst.version = 2;
//		vst.flag = 0;
		vessel->GetStatus(vst);


		target = initialgamma - launchrotate;

		if( target > 180.0 )
			target -= 360.0;
		else if( target < -180.0 )
			target += 360.0;
					
		diff = vst.arot.z*180/PI - target;
		if( diff > 180.0 )
			diff -= 360.0;
		else if( diff < -180.0 )
			diff += 360.0;

		if (MET > 30.0) diff = 0.0;
		if (ROLL < 90.0) diff = 0.0;

		diff = GetBank()*180/PI + target;
		if (ROLL < 90.0) diff = -1.0*GetBank()*180/PI;

		if ((ROLL < 90.0) && (MET < 180)) {
			oapiGetHeading(GetHandle(),&hdg);
			hdg = hdg*180/PI + 180.0;
			if (hdg < 0.0) hdg += 360.0;
			diverge = hdg-PadHeading;
			if (diverge > 35.0) diverge = 35.0;
			if (diverge < -35.0) diverge = -35.0;
			diff += diverge;
		}

		if ((MET > 180) || (ihotol)) {
			diff = -1.0*GetBank()*180/PI;
			yawangle = DEG*GetSlipAngle();
			yawinput = yawangle / 15.0;
			if (ROLL > 90.0) yawinput *= -1.0;
		}
//		sprintf(oapiDebugString(),"hdg %.1f diverge %.1f PadHeading %.1f diff %.1f ",hdg,diverge,PadHeading,diff);

//		sprintf(oapiDebugString(),"initialhdg %.1f initialgamma %.1f launchrotate %.1f target %.1f diff %.1f TGT_ %.1f",initialheading,initialgamma, launchrotate, target,diff, PadHeading);

		targetrate = diff/-1.0;
		if( targetrate > 8.0 )
			targetrate = 8.0;
		else if( targetrate < -8.0 )
			targetrate = -8.0;

		rate = vst.vrot.z * 180/PI;
		amount = fabs (targetrate/8.0);
		if (MET > 0) amount = amount / (MET/10);

//		if (amount > 0.05)
			rollinput = diff/25.0;
//		else
//			rollinput = 0.0;
/*

		if( fabs( rate-targetrate ) > 0.05 ) {
			if( rate < targetrate ) {
//				IncThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.05 );
//				SetThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.0 );
				rollinput = -1*amount;
			}
			else {
//				IncThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.05);
//				SetThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.0 );
				rollinput = amount;
			}
		}
		else {
//			SetThrusterGroupLevel( THGROUP_ATT_BANKLEFT, 0.0 );
//			SetThrusterGroupLevel( THGROUP_ATT_BANKRIGHT, 0.0 );
			rollinput = 0.0;
		}
*/		
//		if ((MET > 60.0) && (diff < 0.5))

//		if (MET > 60.0)
//			Lateral_Guidance(this, SimT, SimT_Last);
//		else
//			yawinput = 0.0;

//		GetAngularVel(AngularVel);

//		yawinput=(-20*AngularVel.y);		

		// END Hendo's roll control


		}

		Altitude=GetAltitude();
		a = ((APO_ALT * 1000 + R_EARTH) + (Altitude + R_EARTH)) / 2;
		e = ((APO_ALT * 1000 + R_EARTH) / a ) - 1;
		h = sqrt(U_EARTH * a * ( 1 - (e * e)));
		V_periapsis = h / (Altitude + R_EARTH);

		
		// Get current velocity
		GetRelativeVel (hEarth, Rel_Vel);
		Velocity = sqrt((Rel_Vel.x*Rel_Vel.x) + (Rel_Vel.y*Rel_Vel.y) + (Rel_Vel.z*Rel_Vel.z));

		if ((MET > 180) && (Velocity > V_periapsis)) 
		{
			MECO_Cond = true;
			TimeMECO = SimT;
			oapiSetEngineLevel(hVessel, ENGINE_MAIN, 0);
			Guidance(false);
			sprintf(oapiDebugString(), "");
			pitchinput = 0.0;
			yawinput = 0.0;
			rollinput = 0.0;
		}

		
	}

	else if (MECO_Cond && PostMECO) 
	{
		if ((SimT - TimeMECO) < 10) {
			sprintf(oapiDebugString(), "Guidance OFF.  Inclination (Equatorial): %.3f", i); 
		}	
		else {
			guidance = false;
			sprintf(oapiDebugString(), "");
		}
	}
}
// END Guidance_Routine()


void HOTOL::GetInclinationSP()
{
	// Gets a new Inclination setting from the pilot
	bool AzInput (void *id, char *str, void *data);
	
//	oapiOpenInputBox ("Inclination Setpoint, e.g. 5163 = 51.63 deg (47 to 56)", IncInput, 0, 25);
	oapiOpenInputBox ("Enter launch azimuth, 0 to 90 (east) to 360", AzInput, 0, 20, (void*)this);

//	GetEquPos( Longitude, Latitude, Radius );
//	sprintf(oapiDebugString(), "Target inclination (Equatorial): %.3f", acos(sin(PadHeading*PI/180)*cos(Latitude))*DEG); 
}


bool AzInput (void *id, char *str, void *data)
{
//	sscanf(str, "%4lf", &PadHeading);
//	PadHeading = PadHeading / 100.0;
//	PadHeading += 180.0;
/*	sscanf(str, "%4lf", &INCLINATION);
	if (INCLINATION > 4700 && INCLINATION < 5600 ) {
		INCLINATION = INCLINATION / 100;
		NewIncSP = true;
	}
*/
	return ((HOTOL*)data)->IncInput (str);
}

bool HOTOL::IncInput (char *rstr)
{
	sscanf(rstr, "%5lf", &PadHeading);
//	PadHeading = PadHeading / 100.0;
	PadHeading += 180.0;
	d_heading = PadHeading;
	displaytime = 0.01;
	return true;
}

// END rjcroy Guidance System

HOTOL::HOTOL (OBJHANDLE hObj, int fmodel)
: COMMON_VESSEL_EX (hObj)
{

	tex = oapiRegisterExhaustTexture("Exhaust_atsme");
	texRCS  = oapiRegisterExhaustTexture ("exhaust_atrcs");

	/* set the propellant handles to NULL so I can check if I need to 
	/* create them in the SetMainStage() function. */
	ph_Hotol=NULL;
	bShieldJettisoned=false;
	bDeorbitFired=false;
	bThrustTerminated=false;
	bEjected=false;
	bEVA=false;
	bAttSet=false;
	bBoosterMade=false;
	bAutoProgrammed=false;
	bControlsMade=false;
	gear_status=DOOR_CLOSED;
	gear_proc=0.0;
	pldoors_status=DOOR_CLOSED;
	pldoors_proc=0.0;
	brakes_status=DOOR_CLOSED;
	brakes_proc=0.0;
	nozzles_status=DOOR_CLOSED;
	nozzles_proc=0.0;
	trolley_status=DOOR_CLOSED;
	trolley_proc=0.0;
	Vold = _V(0,0,0);
	BrakesOn = false;
	bAbort = false;
	AbortChecked = false;
	atchmass = 0.0;
	numengines = 6;
	dt = 0.0;
	simt = 0.0;
	oldsimt = -0.01;
	displaytime = 0.01;
	SimT_Last = 0.0;
	PadHeading = 270.0;
	d_heading = 270.0;
	last_simt=0.0;
	ignition_time=0.0;
	initialgamma = 0.0;
	pitchinput = 0.0;
	yawinput = 0.0;
	rollinput = 0.0;
	stage = 1;
	ihotol = false;

	//		     			   child       pos           dir       rot      handle
	Baypos = CreateAttachment (false, _V(0,4.0,-4.0),   _V(0,0,1), _V(0,-1,0), "X");
	sat_attach = CreateAttachment (false, OFS_SAT, _V(0,1,0), _V(0,0,1), "X");
	Nose_atch1 = CreateAttachment (false, _V(0,-2.55,14.0), _V(0,-0.0675,1), _V(0,1,0), "NG");
	Nose_atch2 = CreateAttachment (true, _V(0,-2.55,14.0), _V(0,-0.0675,1), _V(0,1,0), "NG");

	char classname[30];
	int i;
	strncpy( classname, GetClassName(), 29 );
	classname[29]=0;
	for( i = 0 ; i < strlen( classname ) ; i++ ) {
		classname[i] = toupper(classname[i]);
	}
	if( strstr( classname, "HOTOLWTROLLEY" ) )
		stage = 0;
	else if ( strstr( classname, "INTHOTOL" ) ) 
		ihotol = true;
	else if( strstr( classname, "HOTOL" ) )
		stage = 1;


	Dock0 = CreateDock(_V(0,6.16,-3.0),_V(0,1,0),_V(0,0,1));

	MyID=ConnectToOrbiterSoundDLL3(GetHandle());
//		havesound=false;
//	else
//	{
		havesound=true;
		RequestLoadVesselWave3(MyID,PLAYGEAR,"Sound\\HOTOL\\gear.wav",BOTHVIEW_FADED_FAR);
		RequestLoadVesselWave3(MyID,PLAYBANG,"Sound\\HOTOL\\stg2sep.wav",BOTHVIEW_FADED_FAR);
		RequestLoadVesselWave3(MyID,PLAYJETT,"Sound\\HOTOL\\fasep.wav",BOTHVIEW_FADED_FAR);
		RequestLoadVesselWave3(MyID,PLAYJET,"Sound\\HOTOL\\jet.wav",BOTHVIEW_FADED_FAR);
//	}

	anim_gear = 0.0;
	anim_pldoors = 0.0;
	anim_brakes = 0.0;
	anim_nozzles = 0.0;
	anim_trolley = 0.0;

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GDPS requests. Have it run
	/* in the constructor so it can set up things like the InitFuelLevel
	/* (initial fuel level) variable, and load the payloads */
	CheckCentralMessage();   

/*
	PARTICLESTREAMSPEC rps = {
		0, 20, 20, 0, 0.03, 0.5, 100, 3, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_FLAT, 1, 1, PARTICLESTREAMSPEC::ATM_PLIN, 6e7, 12e7
	};
	AddReentryStream (&rps);
	retex = oapiRegisterReentryTexture("reentry");
	SetReentryTexture(retex,6e6,1,1);
*/

	Firstsweep = true;
	AnimationsMade = false;

}


HOTOL::~HOTOL()
{

}

void HOTOL::MakeAnimations()
{

	AnimationsMade = true;


	hIHOTOL					= oapiLoadMeshGlobal ("IntHOTOL");
	hHOTOL					= oapiLoadMeshGlobal ("HOTOL");
	hHOTOLTk				= oapiLoadMeshGlobal ("HOTOLTrolley");
	hDummy					= oapiLoadMeshGlobal ("dummy");

	
	anim_gear = CreateAnimation (0);
	anim_pldoors = CreateAnimation(0);
	anim_nozzles = CreateAnimation(0);
	anim_elevator = CreateAnimation (0.5);
	anim_elevatortrim = CreateAnimation (0.5);
	anim_raileron = CreateAnimation (0.5);
	anim_laileron = CreateAnimation (0.5);
	anim_rudder = CreateAnimation (0.5);
	anim_brakes = CreateAnimation (0);
	anim_trolley = CreateAnimation (0);

	if (ihotol) {

	static UINT LLGearDoorGrp[1] = {11};
	static MGROUP_ROTATE LLGearDoor (0, LLGearDoorGrp, 1,
		_V(-1.04,0.06,-11.1), _V(0,0,1), (float)( 50*RAD));
	static UINT LGearGrp[2] = {2, 19};
	static MGROUP_ROTATE LGear (0, LGearGrp, 2,
		_V(-2.1,-0.21,-9.7), _V(1,0, 0), (float)(-100*RAD));
	static UINT LGearGrp2[2] = {2, 19};
	static MGROUP_ROTATE LGear2 (0, LGearGrp2, 2,
		_V(-2.1,-0.21,-9.7), _V(0,0, 1), (float)(-15*RAD));

	static UINT RLGearDoorGrp[1] = {12};
	static MGROUP_ROTATE RLGearDoor (0, RLGearDoorGrp, 1,
		_V( 1.04,0.06,-11.1), _V(0,0,1), (float)(-50*RAD));
	static UINT RGearGrp[2] = {1, 18};
	static MGROUP_ROTATE RGear (0, RGearGrp, 2,
		_V( 2.1,-0.21,-9.7), _V(1,0,0), (float)(-100*RAD));
	static UINT RGearGrp2[2] = {1, 18};
	static MGROUP_ROTATE RGear2 (0, RGearGrp2, 2,
		_V( 2.1,-0.21,-9.7), _V(0,0,-1), (float)(-15*RAD));

	static UINT NoseGearDoorGrp[1] = {10};
	static MGROUP_ROTATE NoseGearDoor (0, NoseGearDoorGrp, 1,
		_V(-0.5, 0.5,21.8), _V(0,0.1,1), (float)(-110*RAD));
	static UINT NoseGearGrp[2] = {0, 17};
	static MGROUP_ROTATE NoseGear (0, NoseGearGrp, 2,
		_V(0.0,0.78,20.8), _V(1,0,0), (float)( 92*RAD));


	static UINT TrolleyGrp[2] = {0,1};
	static MGROUP_ROTATE Trolley (1, TrolleyGrp, 2,
		_V(0,-2.736,-7.932), _V(1,0,0), (float)( 45*RAD));

	AddAnimationComponent (anim_trolley, 0.01, 0.99, &Trolley);

// Known Groups:
 
	// 8-12 gone, subtract 5 from 13+
	// 16, 18 gone, subtract 6 from 17, 7 from 19+
// 0 Nose Tire
// 1 R Tire
// 2 L Tire
// 3 HullEtc
// 4 Elevons
// 5 L Ail
// 6 R Ail
// 7 Bay Door
// 8 Rudder
// 9 RCS Quads
// 10 NG Door
// 11 L Lwr MLG Door
// 12 R Lwr MLG Door
// 13 Body Flap
// 14 AftEnd
// 15 Upper Nozzles
// 16 Lower Nozzles
// 17 Nose Strut
// 18 R MLG Strut
// 19 L MLG Strut


	static UINT PLDoorGrp[1] = {7};
	static MGROUP_ROTATE PLDoor (0, PLDoorGrp, 1,
		_V(0,6.4,-7.5), _V( 1,0,0), (float)(-165*RAD));

//	AddAnimationComponent (anim_nozzles, 0.1, 0.9, &Nozl3,Noz1);
//	AddAnimationComponent (anim_nozzles, 0.1, 0.9, &Nozl4,Noz2);


	AddAnimationComponent (anim_gear, 0.05, 0.4, &LLGearDoor);
	AddAnimationComponent (anim_gear, 0.4, 0.9, &LGear);
	AddAnimationComponent (anim_gear, 0.9, 0.95, &LGear2);
	AddAnimationComponent (anim_gear, 0.05, 0.4, &RLGearDoor);
	AddAnimationComponent (anim_gear, 0.4, 0.9, &RGear);
	AddAnimationComponent (anim_gear, 0.9, 0.95, &RGear2);
	AddAnimationComponent (anim_gear, 0.4, 0.9, &NoseGear);
	AddAnimationComponent (anim_gear, 0.1, 0.7, &NoseGearDoor);

	AddAnimationComponent (anim_pldoors, 0.1, 0.9, &PLDoor);

	// ***** Elevator animation *****
	static UINT ElevatorGrp[1] = {4};
	static MGROUP_ROTATE Elevator (0, ElevatorGrp, 1,
		_V(0.0,0.0,-17.48), _V( 1,0,0), (float)( 25*RAD));
	AddAnimationComponent (anim_elevator, 0, 1, &Elevator);

	// ***** Elevator trim animation *****
	static UINT ElevatorTrimGrp[1] = {13};
	static MGROUP_ROTATE ElevatorTrim (0, ElevatorTrimGrp, 1,
		_V(0.0,0.0,-17.48), _V( 1,0,0), (float)( 15*RAD));
	AddAnimationComponent (anim_elevatortrim, 0, 1, &ElevatorTrim);


	// ***** Rudder animation *****
	static UINT RudderGrp[1] = {8};
	static MGROUP_ROTATE Rudder (0, RudderGrp, 1,
		_V( 0,0,18.1), _V( 0,1,0), (float)(-6*RAD));
	AddAnimationComponent (anim_rudder, 0, 1, &Rudder);

	// ***** Aileron animation *****

	static UINT RAileronGrp[1] = {6};
	static MGROUP_ROTATE RAileron (0, RAileronGrp, 1,
		_V(0.0,0.0,-17.48), _V( 1,0,0), (float)( 30*RAD));
	AddAnimationComponent (anim_raileron, 0, 1, &RAileron);

	static UINT LAileronGrp[1] = {5};
	static MGROUP_ROTATE LAileron (0, LAileronGrp, 1,
		_V(0.0,0.0,-17.48), _V( 1,0,0), (float)(-30*RAD));
	AddAnimationComponent (anim_laileron, 0, 1, &LAileron);


	}
	else {
	static UINT LGearDoorGrp[1] = {16};
	static MGROUP_ROTATE LGearDoor (0, LGearDoorGrp, 1,
		_V(-2.5,-1.602,-9.25), _V(0,0,1), (float)(-80*RAD));
	static UINT LLGearDoorGrp[1] = {17};
	static MGROUP_ROTATE LLGearDoor (0, LLGearDoorGrp, 1,
		_V(-1.125,-2.47,-9.25), _V(0,0,1), (float)( 40*RAD));
	static UINT LGearGrp[2] = {2, 26};
	static MGROUP_ROTATE LGear (0, LGearGrp, 2,
		_V(-5.000,-2,-9.25), _V(0,0,1), (float)(-25*RAD));

	static UINT RGearDoorGrp[1] = {18};
	static MGROUP_ROTATE RGearDoor (0, RGearDoorGrp, 1,
		_V( 2.5,-1.602,-9.25), _V(0,0,1), (float)( 80*RAD));
	static UINT RLGearDoorGrp[1] = {19};
	static MGROUP_ROTATE RLGearDoor (0, RLGearDoorGrp, 1,
		_V( 1.125,-2.47,-9.25), _V(0,0,1), (float)(-40*RAD));
	static UINT RGearGrp[2] = {1, 25};
	static MGROUP_ROTATE RGear (0, RGearGrp, 2,
		_V( 5.000,-2,-9.25), _V(0,0,1), (float)( 25*RAD));

	static UINT NoseGearDoorGrp[1] = {15};
	static MGROUP_ROTATE NoseGearDoor (0, NoseGearDoorGrp, 1,
		_V(-0.333, 0.68,37.5), _V(0,0.05,1), (float)(-110*RAD));
	static UINT NoseGearGrp[2] = {0, 24};
	static MGROUP_ROTATE NoseGear (0, NoseGearGrp, 2,
		_V(0.0,0.86,36.5), _V(1,0,0), (float)( 92*RAD));


	static UINT TrolleyGrp[2] = {0,1};
	static MGROUP_ROTATE Trolley (1, TrolleyGrp, 2,
		_V(0,-2.736,-7.932), _V(1,0,0), (float)( 45*RAD));

	AddAnimationComponent (anim_trolley, 0.01, 0.99, &Trolley);

// Known Groups:

// 0 Nose Tire
// 1 R Tire
// 2 L Tire
// 3 HullEtc
// 4 Elevons
// 5 L Ail
// 6 R Ail
// 7 Bay Door
// 8 L FwdRamp
// 9 R FwdRamp
// 10 L AftRamp
// 11 R AftRamp
// 12 Canard
// 13 Rudder
// 14 RCS Quads
// 15 NG Door
// 16 L Upr MLG Door
// 17 L Lwr MLG Door
// 18 R Upr MLG Door
// 19 R Lwr MLG Door
// 20 Body Flap
// 21 AftEnd
// 22 Upper Nozzles
// 23 Lower Nozzles
// 24 Nose Strut
// 25 R MLG Strut
// 26 L MLG Strut


	static UINT PLDoorGrp[1] = {7};
	static MGROUP_ROTATE PLDoor (0, PLDoorGrp, 1,
		_V(0,6.4,-7.5), _V( 1,0,0), (float)(-165*RAD));

	static UINT NozlGrp1[2] = {9,10};
	static MGROUP_ROTATE Nozl1 (0, NozlGrp1, 2,
		_V( 0.163,0,-2.855), _V( 0,1,0), (float)(-16*RAD));
	static UINT NozlGrp2[2] = {8,11};
	static MGROUP_ROTATE Nozl2 (0, NozlGrp2, 2,
		_V(-0.163,0,-2.855), _V( 0,1,0), (float)( 16*RAD));
/*	static UINT NozlGrp3[1] = {10};
	static MGROUP_ROTATE Nozl3 (0, NozlGrp3, 1,
		_V( 0.612,0,-6.021), _V( 0,1,0), (float)( 15*RAD));
	static UINT NozlGrp4[1] = {11};
	static MGROUP_ROTATE Nozl4 (0, NozlGrp4, 1,
		_V(-0.612,0,-6.021), _V( 0,1,0), (float)(-15*RAD));
*/
	Noz1 = AddAnimationComponent (anim_nozzles, 0.1, 0.9, &Nozl1);
	Noz2 = AddAnimationComponent (anim_nozzles, 0.1, 0.9, &Nozl2);
//	AddAnimationComponent (anim_nozzles, 0.1, 0.9, &Nozl3,Noz1);
//	AddAnimationComponent (anim_nozzles, 0.1, 0.9, &Nozl4,Noz2);


	AddAnimationComponent (anim_gear, 0.05, 0.4, &LGearDoor);
	AddAnimationComponent (anim_gear, 0.05, 0.4, &LLGearDoor);
	AddAnimationComponent (anim_gear, 0.4, 0.9, &LGear);
	AddAnimationComponent (anim_gear, 0.05, 0.4, &RGearDoor);
	AddAnimationComponent (anim_gear, 0.05, 0.4, &RLGearDoor);
	AddAnimationComponent (anim_gear, 0.4, 0.9, &RGear);
	AddAnimationComponent (anim_gear, 0.4, 0.9, &NoseGear);
	AddAnimationComponent (anim_gear, 0.1, 0.7, &NoseGearDoor);

	AddAnimationComponent (anim_pldoors, 0.1, 0.9, &PLDoor);

	// ***** Elevator animation *****
	static UINT ElevatorGrp[1] = {4};
	static MGROUP_ROTATE Elevator (0, ElevatorGrp, 1,
		_V(0.0,0.0,-17.48), _V( 1,0,0), (float)( 25*RAD));
	static UINT ElevatorGrp2[1] = {12};
	static MGROUP_ROTATE Elevator2 (0, ElevatorGrp2, 1,
		_V(0.0,1.92, 36.20), _V( 1,0,0), (float)(-25*RAD));
	AddAnimationComponent (anim_elevator, 0, 1, &Elevator);
	AddAnimationComponent (anim_elevator, 0, 1, &Elevator2);

	// ***** Elevator trim animation *****
	static UINT ElevatorTrimGrp[1] = {20};
	static MGROUP_ROTATE ElevatorTrim (0, ElevatorTrimGrp, 1,
		_V(0.0,0.0,-17.48), _V( 1,0,0), (float)( 15*RAD));
	AddAnimationComponent (anim_elevatortrim, 0, 1, &ElevatorTrim);
	static UINT ElevatorTrimGrp2[1] = {12};
	static MGROUP_ROTATE ElevatorTrim2 (0, ElevatorTrimGrp2, 1,
		_V(0.0,1.92, 36.20), _V( 1,0,0), (float)(-15*RAD));
	AddAnimationComponent (anim_elevatortrim, 0, 1, &ElevatorTrim2);

	// ***** Rudder animation *****
	static UINT RudderGrp[1] = {13};
	static MGROUP_ROTATE Rudder (0, RudderGrp, 1,
		_V( 0,0,36.0), _V( 0,1,0), (float)(-6*RAD));
	AddAnimationComponent (anim_rudder, 0, 1, &Rudder);

	// ***** Aileron animation *****

	static UINT RAileronGrp[1] = {6};
	static MGROUP_ROTATE RAileron (0, RAileronGrp, 1,
		_V(0.0,0.0,-17.48), _V( 1,0,0), (float)( 30*RAD));
	AddAnimationComponent (anim_raileron, 0, 1, &RAileron);

	static UINT LAileronGrp[1] = {5};
	static MGROUP_ROTATE LAileron (0, LAileronGrp, 1,
		_V(0.0,0.0,-17.48), _V( 1,0,0), (float)(-30*RAD));
	AddAnimationComponent (anim_laileron, 0, 1, &LAileron);

	}
		// ***** Speedbrake animation *****
/*
//	static UINT RRudderGrp[1] = {12};
	static MGROUP_ROTATE RRudder2 (0, RRudderGrp, 1,
		_V( 9.19,2.0,-9.125), _V( 0.25,0.45,-0.4), (float)(-20*RAD));
//	static UINT LRudderGrp[1] = {13};
	static MGROUP_ROTATE LRudder2 (0, LRudderGrp, 1,
		_V(-9.19,2.0,-9.125), _V(-0.25,0.45,-0.4), (float)( 20*RAD));
*/

//	AddAnimationComponent (anim_brakes, 0, 1, &RRudder2, wing2);
//	AddAnimationComponent (anim_brakes, 0, 1, &LRudder2, wing1);

}


void HOTOL::MakeAirfoils()
{
	ClearAirfoilDefinitions();
	ClearVariableDragElements();

	CreateVariableDragElement (&brakes_proc, 0.8, _V(0, 0, 0));
	CreateVariableDragElement (&gear_proc, 0.6, _V(0, -1, 0));

//                  alignment      position      func      chord S   AR   
	CreateAirfoil (LIFT_VERTICAL, _V(0,0,-0.4), VLiftCoeff, 20, 270, 2.266);
	// wing and body lift+drag components

	CreateAirfoil (LIFT_HORIZONTAL, _V(0,0,-8.5), HLiftCoeff, 3.5, 28, 2.6);
	// vertical stabiliser and body lift and drag components



}


void HOTOL::MakeControlSurfaces()
{

//  Aargh!  Can't delete control surfaces
//  Need to make sure we declare once and only once,
//  but that a booster stage alone doesn't get animations for the upper stage!

	if (bControlsMade) return;
	bControlsMade = true;


		CreateControlSurface (AIRCTRL_ELEVATOR,    16.5,  0.4, _V(  0,0,-11.67), AIRCTRL_AXIS_XPOS, anim_elevator);
		CreateControlSurface (AIRCTRL_RUDDER,       5.3,  0.4, _V(  0,0,-12.75), AIRCTRL_AXIS_YPOS, anim_rudder);
		CreateControlSurface (AIRCTRL_AILERON,     11.3, -0.4, _V(-9.5,0,-12.75), AIRCTRL_AXIS_XPOS, anim_raileron);
		CreateControlSurface (AIRCTRL_AILERON,     11.3, -0.4, _V( 9.5,0,-12.75), AIRCTRL_AXIS_XNEG, anim_laileron);
		CreateControlSurface (AIRCTRL_ELEVATORTRIM, 8.7,  0.5, _V(  0,0,-11.67), AIRCTRL_AXIS_XPOS, anim_elevatortrim);

}


/*
void HOTOL::BoosterSwitchToJets ()
{


	PARTICLESTREAMSPEC contrail = 
	{
		0, 1.1, 30, 325.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::DIFFUSE,
//		0, 1.25, 7, 350, 0.3, 4.0, 7, 3.0, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 25, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};

	SoundOptionOnOff3(MyID,PLAYMAINTHRUST,false);

	SetDefaultPropellantResource (ph_Jets); 

	ClearThrusterDefinitions();
	
	th_first[0] = CreateThruster ( _V( 4.95,0.0,-17.2), _V(0, 0,1), BJET_THRUST, ph_Jets, ISP_BJET_VAC, ISP_BJET_VAC);
	th_first[1] = CreateThruster ( _V( 0.00,0.0,-17.2), _V(0, 0,1), BJET_THRUST, ph_Jets, ISP_BJET_VAC, ISP_BJET_VAC);
	th_first[2] = CreateThruster ( _V(-4.95,0.0,-17.2), _V(0, 0,1), BJET_THRUST, ph_Jets, ISP_BJET_VAC, ISP_BJET_VAC);

	numengines = 3;
	thg_first = CreateThrusterGroup (th_first, 3, THGROUP_MAIN);
//	AddExhaust (th_first[0], 3.0, 0.3, tex);

	AddExhaustStream (th_first[0], _V( 4.95,-2.1,-17.2), &contrail);
	AddExhaustStream (th_first[0], _V( 0.00,-1.8,-17.2), &contrail);
	AddExhaustStream (th_first[0], _V(-4.95,-2.1,-17.2), &contrail);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &exhaust_main);

}
*/
void HOTOL::UpdateAnimations()
{	

		// animate landing gear
		if (gear_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * GEAR_OPERATING_SPEED;
			if (gear_status == DOOR_CLOSING) {
				if (gear_proc > 0.0)
					gear_proc = max (0.0, gear_proc-da);
				else {
					gear_status = DOOR_CLOSED;
				}
			} else  { // door opening
				if (gear_proc < 1.0)
					gear_proc = min (1.0, gear_proc+da);
				else {
					gear_status = DOOR_OPEN;
				}
			}
			SetAnimation (anim_gear, gear_proc);
		}

		// animate payload bay doors
		if (pldoors_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * PLDOORS_OPERATING_SPEED;
						if (pldoors_status == DOOR_CLOSING) {
				if (pldoors_proc > 0.0)
					pldoors_proc = max (0.0, pldoors_proc-da);
				else {
					pldoors_status = DOOR_CLOSED;
				}
			} else  { // door opening
				if (pldoors_proc < 1.0)
					pldoors_proc = min (1.0, pldoors_proc+da);
				else {
					pldoors_status = DOOR_OPEN;
				}
			}
			SetAnimation (anim_pldoors, pldoors_proc);
		}

		// animate speedbrakes
		if (brakes_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * BRAKES_OPERATING_SPEED;
			if (brakes_status == DOOR_CLOSING) {
				if (brakes_proc > 0.0)
					brakes_proc = max (0.0, brakes_proc-da);
				else {
					brakes_status = DOOR_CLOSED;
				}
			} else  { // door opening
				if (brakes_proc < 1.0)
					brakes_proc = min (1.0, brakes_proc+da);
				else {
					brakes_status = DOOR_OPEN;
				}
			}
			SetAnimation (anim_brakes, brakes_proc);
		}

		// animate nozzle extension
/*		if (nozzles_status >= DOOR_CLOSING) {
			double da = oapiGetSimStep() * NOZZLES_OPERATING_SPEED;
			if (nozzles_status == DOOR_CLOSING) {
				if (nozzles_proc > 0.0)
					nozzles_proc = max (0.0, nozzles_proc-da);
				else {
					nozzles_status = DOOR_CLOSED;
				}
			} else  { // door opening
				if (nozzles_proc < 1.0)
					nozzles_proc = min (1.0, nozzles_proc+da);
				else {
					nozzles_status = DOOR_OPEN;
				}
			}
			SetAnimation (anim_nozzles, nozzles_proc);
		}
*/
		SetAnimation (anim_nozzles, nozzles_proc);

		if ((stage == 0) && (GetMachNumber() > 0.1)) {
			trolley_proc = ((GetAOA()*DEG)-0.0)/45.0;
			SetAnimation (anim_trolley, trolley_proc);
		}

}

void HOTOL::ActivateLandingGear (DoorStatus action)
{
	gear_status = action;
}

void HOTOL::RevertLandingGear ()
{
	ActivateLandingGear (gear_status == DOOR_CLOSED || gear_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void HOTOL::ActivatePayloadBayDoors (DoorStatus action)
{
	pldoors_status = action;
}

void HOTOL::RevertPayloadBayDoors ()
{
	ActivatePayloadBayDoors (pldoors_status == DOOR_CLOSED || pldoors_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void HOTOL::ActivateBrakes (DoorStatus action)
{
	brakes_status = action;
}

void HOTOL::RevertBrakes ()
{
	ActivateBrakes (brakes_status == DOOR_CLOSED || brakes_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void HOTOL::ActivateNozzles (DoorStatus action)
{
	nozzles_status = action;
}

void HOTOL::RevertNozzles ()
{
	ActivateNozzles (nozzles_status == DOOR_CLOSED || nozzles_status == DOOR_CLOSING ?
						 DOOR_OPENING : DOOR_CLOSING);
}

void HOTOL::SetGearParameters (double state)
{

	if ((gear_status == DOOR_OPENING) || (gear_status == DOOR_CLOSING)) {
		double geary = (-1*(gear_proc-1.0));
		if (stage==0) {
			SetTouchdownPoints (_V(0,-6.3,36.5), _V(-3.6,-6.7,-13.3), _V(3.6,-6.7,-13.3));
		}
		else if (ihotol) SetTouchdownPoints (_V(0,0.1+geary,36.5), _V(-3.6,-2.0+geary,-9.3), _V(3.6,-2.0+geary,-9.3));
		else {
			SetTouchdownPoints (_V(0,-1.30+geary,36.5), _V(-3.6,-3.7+geary,-9.3), _V(3.6,-3.7+geary,-9.3));
		}
	}
	if (stage==0) SetSurfaceFrictionCoeff (0.02, 0.4);
	else if ((!BrakesOn) && (gear_proc == 1.0)) {
				SetSurfaceFrictionCoeff (0.025, 0.4);
			} else {
				SetSurfaceFrictionCoeff (0.3, 0.4);		// Brakes on!
			}


	double steerforce;
	double steerangle;

	steerforce = 4.0;
	if (GetAirspeed() < 40.0)
		steerforce *=5.0;
	steerforce = steerforce*GetMass();
	if ((GetAirspeed ()-3) < 5.0) steerforce*=((GetAirspeed()-3)/5); // from 1* to 0*
	if (GetAirspeed() < 3.0) steerforce = 0.0;

	steerangle = GetControlSurfaceLevel (AIRCTRL_RUDDER);
	steerforce = steerforce * steerangle;
	if ((GetAltitude() > 4.5) || (gear_proc < 1.0)) steerforce = 0.0;
	AddForce (_V(steerforce, 0, 0), _V(0, 0, 12.0));
	AddForce (_V(-steerforce, 0, 0), _V(0, 0, -12.0));
}

/*
void HOTOL::SeparateEVA ()
{
	VESSELSTATUS vs;
	GetStatus (vs);
	vs.eng_main = vs.eng_hovr = 0.0;
	vs.fuel=1.0;
	vs.status=0;
	VECTOR3 ofs = EVA_OFS;
//	ofs.y += hatch_proc;
	Local2Rel (ofs, vs.rpos);
	char name[256], name2[256];
//	strcpy (name, GetName()); strcat (name, "-PLT");


	UINT i;
	for (i=1;i<8;i++) {
			strcpy (name2, GetName()); 
			sprintf(name, "-EV%d", i);
			strcat (name2, name);
			hvessel=oapiGetVesselByName(name2);
			if (hvessel == 0) break;
	}		
	if ( i < 7) 
	{
		VESSEL::Create (name2, "2001/Astro2001", vs);
		hvessel=oapiGetVesselByName(name2);
		oapiSetFocusObject(hvessel);
	}



//	VESSEL::Create (name, "gemini_leva", vs);
}
*/
void HOTOL::TimeStep(  double simt )
{
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* This function loops through the pending GPDS requests and returns a bool (true/false)
	/* if any GPDS requests have been implemented.
	/* If any have (eg adding a new payload), you will need to re-draw your ship, hence the
	/* call to the (repeatable) SetMainStage(); */
	if( CheckCentralMessage() ) {
		while( CheckCentralMessage() ) ;
			SetMainStage();
	}

	if (!bAutoProgrammed)
		ProgramAuto();
	if (!bControlsMade)
		MakeControlSurfaces();

	dt = simt - oldsimt;
	oldsimt = simt;
	if ((displaytime > 0) && (oapiGetFocusObject() == GetHandle())) displaytime += dt;

	if ((!guidance) && (displaytime < 5) && (stage < 1) && (stage > -3) && (oapiGetFocusObject() == GetHandle()))
	{
		GetEquPos( Longitude, Latitude, Radius );
		sprintf(oapiDebugString(), "Target inclination (Equatorial): %.3f", (180 - acos(sin(d_heading*PI/180)*cos(Latitude))*DEG));
	}
//	else if (oapiGetFocusObject() == GetHandle())
//		sprintf(oapiDebugString(), " ");

	if (!guidance && !MECO_Cond ) {
		if(!NewIncSP) {
			// Set the Inclination setpoint for the guidance computer
			GetEquPos( Longitude, Latitude, Radius );
				INCLINATION = acos(Clairaut_Constant)*DEG;
		}
	}	

	//Check if guidance is on and if so, call
	if (guidance)
	{
/*		if (BadHeading) 
		{
			sprintf(oapiDebugString(),"Guidance system cannot launch from this heading.");
			guidance = false;
		} 
		else 
*/		{
			// Calculate MET
			if (TZero == -100) 
			{
				MET = -5 ;
				TZero = 0;
			}
			else 
			{
//				if (dt > 0)
//					MET += dt;
//				else MET += 0.01;
				MET = simt - TZero;
			}
			// Call the guidance routine
			Guidance_Routine(this, simt, SimT_Last);
			SimT_Last = MET;
		}
	}

	if ((ihotol) && (numengines == 4))					// Throttle back to maintain 5.5 G's
		if (((GetThrusterGroupLevel(THGROUP_MAIN)*THRUST_IHOTOL_VAC*numengines)/GetMass()) > 5.5*G) {
			numengines = 2;
			SetMainStage();
			SetThrusterGroupLevel (THGROUP_MAIN,1.0);
		}
	if ((!ihotol) && (stage > 0) && (numengines == 4))					// Throttle back to maintain 3.0 G's
		if (((GetThrusterGroupLevel(THGROUP_MAIN)*THRUST_FIRST_VAC*numengines)/GetMass()) > 3.0*G) {
			numengines = 2;
			SetMainStage();
			SetThrusterGroupLevel (THGROUP_MAIN,1.0);
		}
	if ((ihotol) && (numengines == 2))					// Throttle back to maintain 5.5 G's
		if (((GetThrusterGroupLevel(THGROUP_MAIN)*THRUST_IHOTOL_VAC*numengines)/GetMass()) > 5.5*G) 
			SetThrusterGroupLevel(THGROUP_MAIN,(GetThrusterGroupLevel(THGROUP_MAIN)*0.98));

	if (numengines == 6) {
		AirBreathingEngine();
		double ThrusterActive = GetThrusterGroupLevel(THGROUP_MAIN);
		if(ThrusterActive>0.0f)
			{
			// we calculate the volume at wich it must be played
			double volume=(110+(145*ThrusterActive))*(1-nozzles_proc);
			double freq=11050+(9000*ThrusterActive);
			PlayVesselWave3(MyID,PLAYJET,LOOP,volume,freq); // play the sound
			}
		else if(IsPlaying3(MyID,PLAYJET))
			{
			// else shut down the sound if it's playing
			StopVesselWave3(MyID,PLAYJET);
			}
	} 	
	else if(IsPlaying3(MyID,PLAYJET))
		{
		// else shut down the sound if it's playing
		StopVesselWave3(MyID,PLAYJET);
		}

	if ((GetAltitude() > 9.0) && (stage == 0))	{		
		JettisonPart();
		SetMainStage();
		SetThrusterGroupLevel (THGROUP_MAIN,1.0);
	}

	UpdateAnimations();
	VectorThrust();
	SetGearParameters(gear_proc);

	if (Firstsweep) {
		AttachSweep();
		Firstsweep = false;
	}
}


void HOTOL::VectorThrust()
{

	if (Firstsweep)
		return;

	if (oapiGetTimeAcceleration() > 10) return;

		if (GetFocus() == GetHandle()) {
			tempP = GetManualControlLevel(THGROUP_ATT_PITCHDOWN, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
				GetManualControlLevel(THGROUP_ATT_PITCHUP, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);

			tempY = GetManualControlLevel(THGROUP_ATT_YAWLEFT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
				GetManualControlLevel(THGROUP_ATT_YAWRIGHT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);

			tempR = GetManualControlLevel(THGROUP_ATT_BANKLEFT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE) - 
				GetManualControlLevel(THGROUP_ATT_BANKRIGHT, MANCTRL_ANYDEVICE, MANCTRL_ANYMODE);

		} else {
			tempP = 0.0;
			tempY = 0.0;
			tempR = 0.0;
		}

		GetAngularVel(AngularVel);

		tempP+=(2*AngularVel.x);			// Bias the thrust vector AGAINST the angular velocity a bit
		tempY+=(-2*AngularVel.y);			// This will damp angular velocity, preventing loss of control
		tempR+=(2*AngularVel.z);
	

		if (stage > 0) {
			tempP+=pitchinput;
			tempY+=yawinput;
			tempR+=rollinput;
		}
		else
			tempP+= -0.5;
	
//	sprintf(oapiDebugString(),"pitchinput, %4f, yawinput, %4f, rollinput, %4f",tempP,tempY,tempR);

		if (ihotol) {
			tempP*=0.1;
			tempY*=0.2;
			tempR*=0.2;
		}

		if (stage < 1) {
			SetThrusterDir(th_first[0],_V(0.02*tempY,0.02*(4*tempP-2*tempR),1));
			SetThrusterDir(th_first[1],_V(0.02*tempY,0.02*(4*tempP+2*tempR),1));
		}
		else if (stage == 1) {
			if (GetMachNumber() < 6.0) {
				SetThrusterDir(th_first[0],_V(0.02*tempY,0.01*numengines*(4*tempP-2*tempR),1));
				SetThrusterDir(th_first[1],_V(0.02*tempY,0.01*numengines*(4*tempP+2*tempR),1));
			} else {
				SetThrusterDir(th_first[0],_V(0.02*tempY,0.002*numengines*(4*tempP-2*tempR),1));
				SetThrusterDir(th_first[1],_V(0.02*tempY,0.002*numengines*(4*tempP+2*tempR),1));
			}
		}
		else if (stage == 2) {
			SetThrusterDir(th_first[0],_V(0.02*tempY,0.02*(4*tempP+2*tempR),1));
			SetThrusterDir(th_first[1],_V(0.02*tempY,0.02*(4*tempP-2*tempR),1));
		}

}

void HOTOL::AirBreathingEngine()
// Thanks to David Hopkins for publishing his AirBreathingEngine source.  
{//=========================================================
	double alt2;
	double eas2;
	double cf1, cf2;
	double ActualThrust;


	if (ihotol) return;

	alt2 = GetAltitude();
	eas2 = GetAirspeed ();

	alt2 = alt2*3.28;
	alt2 = alt2*0.42;		// a kludge that keeps the "airbreathing" engine operating at higher altitude
	eas2 = eas2*2.236;

	cf1 = 1.67e-6*(eas2*eas2)-8.8789e-4*(eas2)+0.9977;	
	cf2 = 3.0e-10*(alt2*alt2)-3.0000e-5*(alt2)+0.9934;

	// Nasty kludge to keep the thrust from increasing dramatically above 100kft
	if (alt2 > 50000)
		cf2 = 0.25*(1-((alt2-50000)*.0001));
	if (cf2 < 0) cf2 = 0;

	if (cf1 > 6.0) cf1 = 6.0;

	if (GetMachNumber() > 6.5) cf1 = 0.0;

	ActualThrust = cf1*cf2*THRUST_JET_VAC;

	if (GetThrusterGroupLevel(THGROUP_MAIN) > 0.05)
		for (int i=0; i < 4; i++)
			SetThrusterLevel (th_first[i],(1.0/(cf1*cf1*cf2*cf2)));  // Reduce the rocket engine power while airbreathing is doing the job

//sprintf(oapiDebugString(),"Actualthrust, %4f, cf1, %4f, cf2, %4f",ActualThrust,cf1,cf2);


	SetThrusterMax0 (th_first[4],ActualThrust);
	SetThrusterMax0 (th_first[5],ActualThrust);

	nozzles_proc = 1.0 - ((cf1*cf2)/GetMachNumber());
	if (nozzles_proc < 0) nozzles_proc = 0.0;
	if (nozzles_proc > 1) nozzles_proc = 1.0;
	SetAnimation (anim_nozzles, nozzles_proc);

	if (GetMachNumber() > 6.5) {
		numengines = 4;
		SetMainStage();
		SetThrusterGroupLevel (THGROUP_MAIN,1.0);
		nozzles_status=DOOR_OPEN;
	}

};


void HOTOL::SetClassCaps(  FILEHANDLE cfg )
{

	CheckCentralMessage();

	SetMainStage();
}

void HOTOL::SaveState( FILEHANDLE scn )
{
	char cbuf[256];

	SaveDefaultState (scn);
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Call this after the normal SaveDefautState() to record attached payloads
	/* it the scenario file */


//	Undock(ALLDOCKS);

	oapiWriteScenario_int (scn, "CONFIGURATION", stage);
	oapiWriteScenario_int (scn, "NUMENGINES", numengines);
	sprintf (cbuf, "%d %0.4f", gear_status, gear_proc);
	oapiWriteScenario_string (scn, "GEAR", cbuf);
	sprintf (cbuf, "%d %0.4f", pldoors_status, pldoors_proc);
	oapiWriteScenario_string (scn, "DOORS", cbuf);
	sprintf (cbuf, "%d %0.4f", brakes_status, brakes_proc);
	oapiWriteScenario_string (scn, "BRAKES", cbuf);
	sprintf (cbuf, "%d %0.4f", nozzles_status, nozzles_proc);
	oapiWriteScenario_string (scn, "NOZZLES", cbuf);
	oapiWriteScenario_float (scn, "TGT_HEADING", PadHeading - 180.0);

	/* save payload lines */
	SaveDefaultStateEx(scn);
}

void HOTOL::LoadState(  FILEHANDLE scn, void *vs )
{
    char *line;

	while (oapiReadScenario_nextline (scn, line)) 
	{
		/* CVE-lite function ! */
		/* IMPORTANT */
		/* Call this for every line in the scenario file. It will automatically
		/* deduce which lines are for it, and load the payloads from the file. */

		/* load payload lines */
		if (!strnicmp (line, "CONFIGURATION", 13)) {
		    sscanf (line+13, "%d", &stage);
		} else if (!strnicmp (line, "NUMENGINES", 10)) {
			sscanf(line+10, "%d", &numengines);
		} else if (!strnicmp (line, "HEADING", 7)) {
			sscanf(line+7, "%lf", &initialheading);
		} else if (!strnicmp (line, "GEAR", 4)) {
		sscanf (line+4, "%d%lf", &gear_status, &gear_proc);
		} else if (!strnicmp (line, "DOORS", 5)) {
		sscanf (line+5, "%d%lf", &pldoors_status, &pldoors_proc);
		} else if (!strnicmp (line, "BRAKES", 6)) {
		sscanf (line+6, "%d%lf", &brakes_status, &brakes_proc);
		} else if (!strnicmp (line, "NOZZLES", 7)) {
		sscanf (line+7, "%d%lf", &nozzles_status, &nozzles_proc);
		} else if( !strncmp( line, "TGT_HEADING", 11 ) ) {
            sscanf (line+11, "%lf", &d_heading);
			PadHeading = d_heading + 180.0;  // It's not pad heading any more; just target heading
			INCLINATION = acos(sin(PadHeading*PI/180)*cos(Latitude))*DEG;
			d_heading = PadHeading;

		} else ParsePayloadString(line);

		ParseScenarioLineEx (line, vs);
	}

	/* always set the stage whenever something may have changed */
	SetMainStage();

	// Double-tap all the animations so they display right
	RevertLandingGear();
	RevertLandingGear();

	RevertBrakes();
	RevertBrakes();

	RevertPayloadBayDoors();
	RevertPayloadBayDoors();

	RevertNozzles();
	RevertNozzles();

	if (brakes_proc == 1.0)
		BrakesOn = true;
//	if (wings_proc > 0.0)
//		MakeANewWing();

	ProgramAuto();

}

void HOTOL::ProgramAuto()
{
		bAutoProgrammed = true;

	if (ihotol) {
		N=8;
		double *fmet = new double[N];
		double *fcpitch = new double[N];

		fmet[0]					=    0.0;
		fmet[1]					=   15.0;
		fmet[2]					=   25.0;
		fmet[3]					=   40.0;
		fmet[4]					=   60.0;
		fmet[5]					=   65.0;
		fmet[6]					=   70.0;
		fmet[7]					=   75.0;

		fcpitch[0]				=  35.0;
		fcpitch[1]				=  28.0;
		fcpitch[2]				=  25.0;
		fcpitch[3]				=  20.0;
		fcpitch[4]				=  15.0;
		fcpitch[5]				=  10.0;
		fcpitch[6]				=   5.0;
		fcpitch[7]				=   0.0;
		ProgramPitchMet( N, fmet, fcpitch );
	}
	else {

/*		fmet[0]					=    0.0;
		fmet[1]					=   35.0;
		fmet[2]					=   65.0;
		fmet[3]					=  215.0;
		fmet[4]					=  235.0;
		fmet[5]					=  660.0;
		fmet[6]					=  690.0;
		fmet[7]					=  710.0;
		fmet[8]					=  725.0;
		fmet[9]					=  745.0;
		fmet[10]				=  750.0;
		fmet[11]				=  762.0;
		fmet[12]				=  767.0;
		fmet[13]				=  770.0;

		fcpitch[0]				=  5.0;
		fcpitch[1]				=  8.0;
		fcpitch[2]				=  5.0;
		fcpitch[3]				=  6.0;
		fcpitch[4]				= 28.0;
		fcpitch[5]				= 25.0;
		fcpitch[6]				= 22.0;
		fcpitch[7]				= 20.0;
		fcpitch[8]				= 18.0;
		fcpitch[9]				= 15.0;
		fcpitch[10]				= 10.0;
		fcpitch[11]				=  5.0;
		fcpitch[12]				=  2.5;
		fcpitch[13]				=  0.0;
*/
		N=14;
		double *fmet = new double[N];
		double *fcpitch = new double[N];

		fmet[0]					=    0.0;
		fmet[1]					=   15.0;
		fmet[2]					=   65.0;
		fmet[3]					=  135.0;
		fmet[4]					=  145.0;
		fmet[5]					=  180.0;
		fmet[6]					=  200.0;
		fmet[7]					=  220.0;
		fmet[8]					=  245.0;
		fmet[9]					=  285.0;
		fmet[10]				=  320.0;
		fmet[11]				=  372.0;
		fmet[12]				=  377.0;
		fmet[13]				=  380.0;

		fcpitch[0]				=  10.0;
		fcpitch[1]				=  20.0;
		fcpitch[2]				=  14.0;
		fcpitch[3]				=  10.0;
		fcpitch[4]				=  25.0;
		fcpitch[5]				=  23.0;
		fcpitch[6]				=  22.0;
		fcpitch[7]				=  20.0;
		fcpitch[8]				=  18.0;
		fcpitch[9]				=  15.0;
		fcpitch[10]				=  10.0;
		fcpitch[11]				=   5.0;
		fcpitch[12]				=   2.5;
		fcpitch[13]				=   0.0;
		ProgramPitchMet( N, fmet, fcpitch );
	}
}


void HOTOL::PlaceComponents()
{
	COMMON_MESH tmp;
/*
	tmp.MeshHandle(hSeat);
	tmp.MeshPosition(EJSEAT_OFS);
	AddMeshEx (tmp);
	tmp.MeshHandle(hSeat2);
	tmp.MeshPosition(EJSEAT_OFS2);
	AddMeshEx (tmp);

	tmp.MeshHandle(hAstro);
	tmp.MeshPosition(ASTRO_OFS);
	AddMeshEx (tmp);
	tmp.MeshHandle(hAstro2);
	tmp.MeshPosition(ASTRO_OFS2);
	AddMeshEx (tmp);
*/

	if (ihotol) {
		tmp.MeshHandle(hIHOTOL);
		tmp.MeshPosition(HOTOL_OFS);
		AddMeshEx (tmp);
	}
	else {
		tmp.MeshHandle(hHOTOL);
		tmp.MeshPosition(HOTOL_OFS);
		AddMeshEx (tmp);
	}

	if (stage == 0)  {
		tmp.MeshHandle(hHOTOLTk);
		tmp.MeshPosition(TANK_OFS);
		AddMeshEx (tmp);
	}
}

/* IMPORTANT */
/* Always have a function that you can call as many times as you like that will re-draw
/* all you meshes, reset all your thrusters and check to make sure the propellent is
/* there. This should be called after any configuration change, or any GDPS line (see
/* TimeStep(), above) */

void HOTOL::SetMainStage( )
{

	if (!AnimationsMade)
		MakeAnimations();

	if (ihotol)
		SetIHOTOLStage();
	else if (stage == 0)
		SetTrolleyStage();
	else  
		SetHOTOLStage();
}


void HOTOL::SetTrolleyStage( )
{


	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 1.1, 7, 25.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 0.1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 7, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");
	SetSize (20);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	SetEmptyMass ( HOTOL_DRYMASS + TANK_DRYMASS + atchmass + CalculateMass() );

	SetTouchdownPoints (_V(0,-6.3,36.5), _V(-3.6,-6.7,-13.3), _V(3.6,-6.7,-13.3));
	SetSurfaceFrictionCoeff (0.025, 0.4);
	if (!Nose_atch1) Nose_atch1 = CreateAttachment (false, _V(0,-4.05,14.0), _V(0,-0.0075,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch1, _V(0,-4.05,14.0), _V(0,-0.0075,1), _V(0,1,0));
	if (!Nose_atch2) Nose_atch2 = CreateAttachment (true, _V(0,-4.05,14.0), _V(0,-0.0075,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch2, _V(0,-4.05,14.0), _V(0,-0.0075,1), _V(0,1,0));
	SetPMI (_V(18,19.5,3.25));
	SetCrossSections (_V(234.8,389.1,58.2));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.025);
	SetCameraOffset(CAMERA_OFS);
	MakeAirfoils();
	SetControlSurfaceLevel(AIRCTRL_ELEVATORTRIM,0.7);	
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();
//	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);
//	SetMeshVisibilityMode(1,MESHVIS_ALWAYS);


	/* we're not using COMMON_MESH today, so draw our own */
//	AddMesh(hHOTOL, &VECTOR_ZERO );
	

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */

	if (!ph_Hotol)  
		ph_Hotol  = CreatePropellantResource(SECOND_PROP_MASS,SECOND_PROP_MASS,1.0); 
//	if (!ph_Hotol)  
//		ph_Hotol  = CreatePropellantResource(RCS_PROP_MASS,RCS_PROP_MASS,1.0); 

	SetDefaultPropellantResource (ph_Hotol); 

	th_first[0] = CreateThruster ( _V(-1.81,0,-18.85), _V(0,0,1), THRUST_FIRST_VAC, ph_Hotol, ISP_FIRST_VAC, ISP_FIRST_SL);
	AddExhaustStream (th_first[0], _V(-1.81,1.6,-18.85), &contrail);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &exhaust_main);

	th_first[1] = CreateThruster ( _V( 1.81,0,-18.85), _V(0,0,1), THRUST_FIRST_VAC, ph_Hotol, ISP_FIRST_VAC, ISP_FIRST_SL);
	AddExhaustStream (th_first[1], _V( 1.81,1.6,-18.85), &contrail);
//	AddExhaustStream (th_first[1], _V(-0.72,0,-7), &exhaust_main);

	th_first[2] = CreateThruster ( _V(-1.81,0,-19.1), _V(0,0,1), THRUST_FIRST_VAC, ph_Hotol, ISP_FIRST_VAC, ISP_FIRST_SL);
	AddExhaustStream (th_first[2], _V(-1.81,4.51,-19.1), &contrail);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &exhaust_main);

	th_first[3] = CreateThruster ( _V( 1.81,0,-19.1), _V(0,0,1), THRUST_FIRST_VAC, ph_Hotol, ISP_FIRST_VAC, ISP_FIRST_SL);
	AddExhaustStream (th_first[3], _V( 1.81,4.51,-19.1), &contrail);
//	AddExhaustStream (th_first[1], _V(-0.72,0,-7), &exhaust_main);

	th_first[4] = CreateThruster ( _V(-1.2,-0,-13.0), _V(0,0,1), THRUST_JET_VAC, ph_Hotol, ISP_JET_VAC, ISP_JET_VAC);
	AddExhaustStream (th_first[4], _V(-1.2,-1.35,-13.0), &contrail);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &exhaust_main);

	th_first[5] = CreateThruster ( _V( 1.2,-0,-13.0), _V(0,0,1), THRUST_JET_VAC, ph_Hotol, ISP_JET_VAC, ISP_JET_VAC);
	AddExhaustStream (th_first[5], _V( 1.2,-1.35,-13.0), &contrail);
//	AddExhaustStream (th_first[1], _V(-0.72,0,-7), &exhaust_main);

	thg_first = CreateThrusterGroup (th_first, 6, THGROUP_MAIN);

	AddExhaust (th_first[0], 20.0, 2.0, _V(-1.81,1.6,-18.85), _V(0,0,-1), tex);
	AddExhaust (th_first[1], 20.0, 2.0, _V( 1.81,1.6,-18.85), _V(0,0,-1), tex);
	AddExhaust (th_first[2], 20.0, 2.0, _V(-1.81,4.51,-19.1), _V(0,0,-1), tex);
	AddExhaust (th_first[3], 20.0, 2.0, _V( 1.81,4.51,-19.1), _V(0,0,-1), tex);
	numengines = 6;

	EnableTransponder(true);
	SetAttitudeMode(0);
}


void HOTOL::SetHOTOLStage( )
{


	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 1.1, 7, 25.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 0.1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 7, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");

	SetSize (17);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	SetEmptyMass ( HOTOL_DRYMASS + atchmass + CalculateMass() );


	SetTouchdownPoints (_V(0,-1.30,36.5), _V(-3.6,-3.7,-9.3), _V(3.6,-3.7,-9.3));
	if (!Nose_atch1) Nose_atch1 = CreateAttachment (false, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch1, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0));
	if (!Nose_atch2) Nose_atch2 = CreateAttachment (true, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch2, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0));
	SetPMI (_V(16.7,18.9,3.6));
	SetCrossSections (_V(234.8,389.1,58.2));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.025);
	SetCameraOffset(CAMERA_OFS);
	MakeAirfoils();
//	SetMeshVisibilityMode(3,MESHVIS_ALWAYS);

	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();
//	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);

	/* we're not using COMMON_MESH today, so draw our own */
//	AddMesh(hHOTOL, &VECTOR_ZERO );
	

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */

	if (ph_Hotol==NULL)  
		ph_Hotol  = CreatePropellantResource(SECOND_PROP_MASS,SECOND_PROP_MASS,1.0); 

	SetDefaultPropellantResource (ph_Hotol); 

	th_first[0] = CreateThruster ( _V(-1.81,0,-18.85), _V(0,0,1), THRUST_FIRST_VAC, ph_Hotol, ISP_FIRST_VAC, ISP_FIRST_SL);
	AddExhaustStream (th_first[0], _V(-1.81,1.6,-18.85), &contrail);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &exhaust_main);

	th_first[1] = CreateThruster ( _V( 1.81,0,-18.85), _V(0,0,1), THRUST_FIRST_VAC, ph_Hotol, ISP_FIRST_VAC, ISP_FIRST_SL);
	AddExhaustStream (th_first[1], _V( 1.81,1.6,-18.85), &contrail);
//	AddExhaustStream (th_first[1], _V(-0.72,0,-7), &exhaust_main);

	if (numengines > 2) {
		th_first[2] = CreateThruster ( _V(-1.81,0,-19.1), _V(0,0,1), THRUST_FIRST_VAC, ph_Hotol, ISP_FIRST_VAC, ISP_FIRST_SL);
		AddExhaustStream (th_first[2], _V(-1.81,4.51,-19.1), &contrail);
	//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &exhaust_main);

		th_first[3] = CreateThruster ( _V( 1.81,0,-19.1), _V(0,0,1), THRUST_FIRST_VAC, ph_Hotol, ISP_FIRST_VAC, ISP_FIRST_SL);
		AddExhaustStream (th_first[3], _V( 1.81,4.51,-19.1), &contrail);
	//	AddExhaustStream (th_first[1], _V(-0.72,0,-7), &exhaust_main);
	}

	if (numengines > 4) {
		th_first[4] = CreateThruster ( _V(-1.2,-0,-13.0), _V(0,0,1), THRUST_JET_VAC, ph_Hotol, ISP_JET_VAC, ISP_JET_VAC);
		AddExhaustStream (th_first[4], _V(-1.2,-1.35,-13.0), &contrail);
	//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &exhaust_main);

		th_first[5] = CreateThruster ( _V( 1.2,-0,-13.0), _V(0,0,1), THRUST_JET_VAC, ph_Hotol, ISP_JET_VAC, ISP_JET_VAC);
		AddExhaustStream (th_first[5], _V( 1.2,-1.35,-13.0), &contrail);
	//	AddExhaustStream (th_first[1], _V(-0.72,0,-7), &exhaust_main);
	}
	thg_first = CreateThrusterGroup (th_first, numengines, THGROUP_MAIN);

	AddExhaust (th_first[0], 20.0, 2.0, _V(-1.81,1.6,-18.85), _V(0,0,-1), tex);
	AddExhaust (th_first[1], 20.0, 2.0, _V( 1.81,1.6,-18.85), _V(0,0,-1), tex);
	if (numengines > 2) {
		AddExhaust (th_first[2], 20.0, 2.0, _V(-1.81,4.51,-19.1), _V(0,0,-1), tex);
		AddExhaust (th_first[3], 20.0, 2.0, _V( 1.81,4.51,-19.1), _V(0,0,-1), tex);
	}

	SetAttControlsHOTOL();
	EnableTransponder(true);

}


void HOTOL::SetIHOTOLStage( )
{


	/* CVE-lite function ! */
	/* ClearMeshesEx() will get rid of all the normal meshes, plus anything you've defined
	/* with the COMMON_MESH class (not used here) */

	/* erase all meshes and thrusters so we can start afresh */
	ClearMeshesEx();
	ClearThrusterDefinitions();

	PARTICLESTREAMSPEC contrail = 
	{
		0, 1.1, 7, 25.0, 0, 0.5, -.5, 0.5, PARTICLESTREAMSPEC::DIFFUSE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 0.1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-4, 1
	};

	PARTICLESTREAMSPEC exhaust_main = {
		0, 3, 7, 350, 0.1, 0.2, 28, 1.0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLOG, 1e-5, 0.1
	};
	exhaust_main.tex = oapiRegisterParticleTexture ("Contrail2");

	SetSize (17);

	/* CVE-lite function ! */
	/* IMPORTANT */
	/* double CalculateMass() will return the all-up mass of the combined payloads and
	/* upper stages. Simply add this to your normal dry mass. */
	
	/* always add payload mass [CalculateMass()] to the vessel's empty mass */
	SetEmptyMass ( IHOTOL_DRYMASS + atchmass + CalculateMass() );


	SetTouchdownPoints (_V(0,0.1,36.5), _V(-3.6,-2.0,-9.3), _V(3.6,-2.0,-9.3));
	if (!Nose_atch1) Nose_atch1 = CreateAttachment (false, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch1, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0));
	if (!Nose_atch2) Nose_atch2 = CreateAttachment (true, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0), "NG");
		else SetAttachmentParams (Nose_atch2, _V(0,-2.55,14.0), _V(0,0.0675,1), _V(0,1,0));
	SetPMI (_V(16.7,18.9,3.6));
	SetCrossSections (_V(234.8,389.1,58.2));
	SetCW (0.1, 0.3, 1.4, 1.4);
	SetRotDrag (_V(0.7,0.7,1.2));
	SetPitchMomentScale (1e-6);
	SetBankMomentScale (1e-6);
//	SetLiftCoeffFunc (0); 
	SetTrimScale (0.025);
	SetCameraOffset(ICAMERA_OFS);
	MakeAirfoils();
	SetControlSurfaceLevel(AIRCTRL_ELEVATORTRIM,1.0);
//	SetMeshVisibilityMode(3,MESHVIS_ALWAYS);
	if (numengines > 4)
		numengines = 4;
	
	/* CVE-lite function ! */
	/* IMPORTANT */
	/* Draw() should be called to render all the loaded payloads, plus anything you've
	/* added with the COMMON_MESH class (not used here) */

	/* call draw to render the payloads, plus anything you've added from COMMON_MESH */
	PlaceComponents();
	Draw();
//	SetMeshVisibilityMode(0,MESHVIS_ALWAYS);

	/* we're not using COMMON_MESH today, so draw our own */
//	AddMesh(hHOTOL, &VECTOR_ZERO );
	

	/* CVE-lite variable ! */
	/* IMPORTANT */
	/* Use the keyword double InitFuelLevel multipled by the maximum mass to have the
	/* fuel levels automatically set from the GPDS payload string */

	/* verify that the propellent hasn't been created before making it */

	if (ph_Hotol==NULL)  
		ph_Hotol  = CreatePropellantResource(IHOTOL_PROP_MASS,IHOTOL_PROP_MASS,1.0); 

	SetDefaultPropellantResource (ph_Hotol); 

	th_first[0] = CreateThruster ( _V(-1.81,0,-18.85), _V(0,0,1), THRUST_IHOTOL_VAC, ph_Hotol, ISP_IHOTOL_VAC, ISP_IHOTOL_SL);
	AddExhaustStream (th_first[0], _V(-1.81,1.6,-18.85), &contrail);
//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &exhaust_main);

	th_first[1] = CreateThruster ( _V( 1.81,0,-18.85), _V(0,0,1), THRUST_IHOTOL_VAC, ph_Hotol, ISP_IHOTOL_VAC, ISP_IHOTOL_SL);
	AddExhaustStream (th_first[1], _V( 1.81,1.6,-18.85), &contrail);
//	AddExhaustStream (th_first[1], _V(-0.72,0,-7), &exhaust_main);

	if (numengines > 2) {
		th_first[2] = CreateThruster ( _V(-1.81,0,-19.1), _V(0,0,1), THRUST_IHOTOL_VAC, ph_Hotol, ISP_IHOTOL_VAC, ISP_IHOTOL_SL);
		AddExhaustStream (th_first[2], _V(-1.81,4.51,-19.1), &contrail);
	//	AddExhaustStream (th_first[0], _V(0.72,0,-7), &exhaust_main);

		th_first[3] = CreateThruster ( _V( 1.81,0,-19.1), _V(0,0,1), THRUST_IHOTOL_VAC, ph_Hotol, ISP_IHOTOL_VAC, ISP_IHOTOL_SL);
		AddExhaustStream (th_first[3], _V( 1.81,4.51,-19.1), &contrail);
	//	AddExhaustStream (th_first[1], _V(-0.72,0,-7), &exhaust_main);
	}

	thg_first = CreateThrusterGroup (th_first, numengines, THGROUP_MAIN);

	AddExhaust (th_first[0], 20.0, 2.0, _V(-1.81,1.6,-18.85), _V(0,0,-1), tex);
	AddExhaust (th_first[1], 20.0, 2.0, _V( 1.81,1.6,-18.85), _V(0,0,-1), tex);
	if (numengines > 2) {
		AddExhaust (th_first[2], 20.0, 2.0, _V(-1.81,4.51,-19.1), _V(0,0,-1), tex);
		AddExhaust (th_first[3], 20.0, 2.0, _V( 1.81,4.51,-19.1), _V(0,0,-1), tex);
	}

	SetAttControlsIHOTOL();
	EnableTransponder(true);
	if (GetAltitude()<60000)
		SetAttitudeMode(0);

}


/** Handle buffered key events sent to the vessel by orbiter. 
 *  @param dwKey The scan code of the key
 *  @param fPressed true if pressed, false if depressed
 *  @param keystate the current state of the keyboard
 *
 *  modified switch routine by dave rowbotham.
 */

void HOTOL::JettisonPart()
 {

				VESSELSTATUS vs;
				VECTOR3 ofs = _V(0,0,0);
				VECTOR3 vel = _V(0,-4.5,6.0);
//				ofs = Centaur_OFS;
				GetStatus (vs);
				vs.eng_main = vs.eng_hovr = 0.0;
//				SetEngineLevel(ENGINE_MAIN, 0);
				VECTOR3 rofs, rvel = {vs.rvel.x, vs.rvel.y, vs.rvel.z};
				Local2Rel (ofs, vs.rpos);
				GlobalRot (vel, rofs);
				vs.rvel.x = rvel.x+rofs.x;
				vs.rvel.y = rvel.y+rofs.y;
				vs.rvel.z = rvel.z+rofs.z;
				vs.vrot.x = -0.5;
				vs.vrot.y = 0.0;
				vs.vrot.z = 0.0;
				char name[256];
				if (stage == 0) {
					SetAnimation (anim_trolley, 0.0);
					DelAnimation (anim_trolley);
					strcpy (name, GetName()); strcat (name, "-Trolley");
					VESSEL::Create (name, "HOTOLTrolley", vs);	
					OBJHANDLE hvessel;
					hvessel=oapiGetVesselByName(name);
					if (hvessel != 0) {
						VESSEL *remotevessel = oapiGetVesselInterface(hvessel);					
						remotevessel->SetSurfaceFrictionCoeff (0.45, 0.45);		// Brakes on!
						anim_trolley2 = remotevessel->CreateAnimation (0);		// Use animation to raise the arm
						static UINT TrolleyGrp2[1] = {2};						// of the dropped trolley to match our AOA
						static MGROUP_ROTATE Trolley2 (0, TrolleyGrp2, 1,
							_V(0,-2.736,-7.932), _V(1,0,0), (float)(-45*RAD));
						remotevessel->AddAnimationComponent (anim_trolley2, 0.01, 0.99, &Trolley2);
						remotevessel->SetAnimation (anim_trolley2, trolley_proc);
					}
					trolley_proc = 0.0;
				}
/*				else if (GetThrusterGroupLevel (thg_first) == 0.0) {
					strcpy (name, GetName()); strcat (name, "-Deorbit Stage");
					VESSEL::Create (name, "HOTOL/HOTOL_Deorbit", vs);
					DelPropellantResource (ph_deorbit); 
					PlayVesselWave3(MyID,PLAYBANG);
				}
*/
//				else if (stage == 1) {
//					DelPropellantResource (ph_deorbit);
//				}
				else if (stage > 0) stage--;


				stage++;

 }

void HOTOL::AttachSweep()
{

	
			/* Now scan for nearby attach points; these need to stay in the bay!  */
				
			Local2Global (_V(0,4.0,-4.0), grms);  // global position of RMS tip
			
			// Search the complete vessel list for a grappling candidate.
			// Not very scalable ...

			for (i = 0; i < oapiGetVesselCount(); i++) {
				hV = oapiGetVesselByIndex (i);
				if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
				oapiGetGlobalPos (hV, &gpos);
//				sprintf(oapiDebugString(),"Distance is, %d, Too far to strap in!",dist(gpos, grms));
				if (dist (gpos, grms) < oapiGetSize (hV)) { // in range
					VESSEL *v = oapiGetVesselInterface (hV);
					DWORD nAttach = v->AttachmentCount (true);
					for (DWORD j = 0; j < nAttach; j++) { // now scan all attachment points of the candidate
						ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
						const char *id = v->GetAttachmentId (hAtt);
						if (strncmp (id, "BOLT", 4)) continue; // attachment point not compatible
						v->GetAttachmentParams (hAtt, pos, dir, rot);
						v->Local2Global (pos, gpos);
						if (dist (gpos, grms) < 0.25) { // found one!
							// check whether satellite is currently clamped into payload bay
							if ((hV == GetAttachmentStatus (Baypos)) && (!Firstsweep))
							{
								AttachChild (hV, Baypos, hAtt);
								atchmass = v->GetMass();
								BayVessel = hV;
								BayV = oapiGetVesselInterface (BayVessel);
							}
							else
							{	
								AttachChild (hV, Baypos, hAtt);
								atchmass = v->GetMass();
								hDock = v->GetDockHandle(0);
								BayVessel = hV;
								BayV = oapiGetVesselInterface (BayVessel);
								if (hDock != NULL) {
									v->GetDockParams(hDock,pos, dir, rot);
//									pos.z += -4.5;
//									CreateDock(pos, dir, rot);
									SetDockParams(Dock0, pos, dir, rot);
									v->DelDock(hDock);
								}
								else {
									Undock(ALLDOCKS);
									DelDock(Dock0);
								}
							}
//							return 1;
						}
					}
				}
			}
}

int HOTOL::ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate)
{
	if (!fPressed) return 0; // only process keydown events
	if (KEYMOD_SHIFT (keystate)) 
	{
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		// insert ctrl key combinations here
	}
	else {

		if (KEYDOWN (keystate, OAPI_KEY_J)) 
		{ // "Jettison Stage"
			if (oapiAcceptDelayedKey (OAPI_KEY_J, 1.0))
			{
				VESSEL *depV;
							if ((stage < 2) && (pldoors_proc == 0.0)) {
								JettisonPart();
							}
							else {
								OverridePayloadVelocity(_V(0,+0.25,0));
								depV = DeployPayload();
								if (depV == NULL) {
									if (GetAttachmentStatus (sat_attach) != NULL)
										DetachChild (sat_attach);
									else {
											for (DWORD i = 0; i < oapiGetVesselCount(); i++) {
											OBJHANDLE hV = oapiGetVesselByIndex (i);
											if (hV == GetHandle()) continue; // we don't want to grapple ourselves ...
											oapiGetGlobalPos (hV, &gpos);
											if (dist (gpos, grms) < oapiGetSize (hV)) { // in range
												VESSEL *v = oapiGetVesselInterface (hV);
												DWORD nAttach = v->AttachmentCount (true);
												for (DWORD j = 0; j < nAttach; j++) { // now scan all attachment points of the candidate
													ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
													const char *id = v->GetAttachmentId (hAtt);
													if (strncmp (id, "GS", 2)) continue; // attachment point not compatible
													v->GetAttachmentParams (hAtt, pos, dir, rot);
													v->Local2Global (pos, gpos);
													if (dist (gpos, grms) < MAX_GRAPPLING_DIST) { // found one!
														// check whether satellite is currently clamped into payload bay
														if (hV == GetAttachmentStatus (sat_attach))
															DetachChild (sat_attach);
														AttachChild (hV, sat_attach, hAtt);
														}
													}
												}
											}
									}
								}
							}


							/* CVE-lite function ! */
							/* IMPORTANT */
							/* call DeployPayload() to jettison the GPDS payloads in sequence */

							AttachSweep();

							/* IMPORTANT */
							/* Always re-set the stage after any kind of change */
							SetMainStage();
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_G)) 
		{ // Toggle the Gear
			if (oapiAcceptDelayedKey (OAPI_KEY_G, 1.0))
			{
				RevertLandingGear ();
				StopVesselWave3(MyID,PLAYGEAR);
				PlayVesselWave3(MyID,PLAYGEAR);
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_K)) 
		{ // Toggle the bay doors
			if ((oapiAcceptDelayedKey (OAPI_KEY_K, 1.0)) && (stage > -3))
			{
				RevertPayloadBayDoors ();
//				if (BayVessel != NULL)
//					SendCentralMessage(BayV, "MSG", "REVERTPROTRUSIONS");
//				if (pldoors_status == DOOR_CLOSING)
//					Undock(ALLDOCKS);
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_B)) 
		{ // Toggle the brakes
			if (oapiAcceptDelayedKey (OAPI_KEY_B, 1.0))
			{
//				if (BrakesOn)
//					BrakesOn = false;
//				else BrakesOn = true;
				RevertBrakes();
				if (brakes_status == DOOR_OPENING)
					BrakesOn = true;
				else BrakesOn = false;
			}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_O))
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_O, 1.0))
				if (!guidance) { 
					Guidance( true );
//					guidance=true;
					sprintf(oapiDebugString(),"Guidance ON");
					pitchinput = 0.0;
					yawinput = 0.0;
					rollinput = 0.0;
				}
				else {
					Guidance( false );
//					guidance = false;
					SetAttitudeRotLevel(_V(0,0,0));
					pitchinput = 0.0;
					yawinput = 0.0;
					rollinput = 0.0;

					sprintf(oapiDebugString(),"Guidance OFF");
				}
			return 1;
		}
		if (KEYDOWN (keystate, OAPI_KEY_U)) 
		{ // "Set launch Azimuth"
			if (oapiAcceptDelayedKey (OAPI_KEY_U, 1.0))
			{
				GetInclinationSP();
				return 1;
			}
		}
/*		if (KEYDOWN (keystate, OAPI_KEY_E)) 
		{ 
			if (oapiAcceptDelayedKey (OAPI_KEY_E, 1.0))
				{
				if (hatch_status == DOOR_OPEN)
					{
		//				char name[256];
		//				OBJHANDLE hvessel;
		//				strcpy (name, GetName()); strcat (name, "-EV1");
		//				hvessel=oapiGetVesselByName(name);
		//				if (hvessel == 0)
		//				{
		//					bEVA = true;
							SeparateEVA();
							SetMainStage();
		//					hvessel=oapiGetVesselByName(name);
		//					if (hvessel != 0)
		//						oapiSetFocusObject(hvessel);
		//				}
		//				else
		//				{
		//					oapiDeleteVessel(hvessel);
		//					bEVA = false;
		//					SetMainStage();
		//				};
					}
			}
		return 1;
		}
*/
	}

	return 0;
}




int HOTOL::ConsumeKey( const char *keystate )
{

	if (KEYMOD_SHIFT (keystate)) 
	{
						
		return 0; // shift combinations are reserved
	}
	else if (KEYMOD_CONTROL (keystate)) 
	{
		return 0;
	}
	else 
	{ // unmodified keys
	
		return 0;

	}
	return 0;
}




void HOTOL::SetAttControlsHOTOL()
{
	bAttSet = true;
	THRUSTER_HANDLE th_rot[2];

	th_rot[0] = CreateThruster (_V(0,-0.97, 5.25), _V(0, 1,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.97,-5.25), _V(0,-1,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-0.6,1.97, 40.95), _V(-0.707,-0.707,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 0.6,1.97, 40.95), _V( 0.707,-0.707,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.01,3.41,-17.21), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.01,3.41,-17.21), _V(0, 1.0,0.0),texRCS);  
	thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);

	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,1,0),MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,1,0),MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-0.6,1.97, 40.95), _V(-0.707,-0.707,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 0.6,1.97, 40.95), _V( 0.707,-0.707,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.04,2.72,-17.21), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.04,2.72,-17.21), _V(0,-1.0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_UP);

	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,-1,0),MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,-1,0),MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-0.6,1.97, 40.95), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 0.6,1.97, 40.95), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.01,3.41,-17.21), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.01,3.41,-17.21), _V(0, 1.0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_DOWN);

	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0,-1,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0, 1,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-0.6,1.97, 40.95), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 0.6,1.97, 40.95), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.04,2.72,-17.21), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.04,2.72,-17.21), _V(0,-1.0,0.0),texRCS);  
	thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V(-1,0,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V( 1,0,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 0.6,1.97, 40.95), _V( 1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.24,3.22,-17.21), _V(-1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.24,2.94,-17.21), _V(-1.0,0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V( 1,0,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V(-1,0,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-0.6,1.97, 40.95), _V(-1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.24,3.22,-17.21), _V( 1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.24,2.94,-17.21), _V( 1.0,0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);

	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(-1,0,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 0.6,1.97, 40.95), _V( 1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.24,3.22,-17.21), _V( 1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.24,2.94,-17.21), _V( 1.0,0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_LEFT);
	th_rot[0] = CreateThruster (_V(-2,0.0,0.0), _V( 1,0,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-0.6,1.97, 40.95), _V(-1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.24,3.22,-17.21), _V(-1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.24,2.94,-17.21), _V(-1.0,0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_RIGHT);

	th_rot[0] = CreateThruster (_V(13,0,0), _V(0,-1,0), MAX_ATT_FIRST/4, ph_Hotol, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, 1,0), MAX_ATT_FIRST/4, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.04,2.72,-17.21), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.01,3.41,-17.21), _V(0, 1.0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);

	th_rot[0] = CreateThruster (_V(13,0, 0), _V(0, 1,0), MAX_ATT_FIRST/4, ph_Hotol, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, -1,0), MAX_ATT_FIRST/4, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.04,2.72,-17.21), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.01,3.41,-17.21), _V(0, 1.0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);

	th_rot[0] = CreateThruster (_V(0,0, -4), _V(0,0,1), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.047, 3.08,-17.47), _V(0,0,-1),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.047, 3.08,-17.47), _V(0,0,-1),texRCS);  
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_FORWARD);
	th_rot[0] = CreateThruster (_V(0,0, 4), _V(0,0,-1), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 0.11, 1.97, 41.37), _V(0,0,1),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-0.11, 1.97, 41.37), _V(0,0,1),texRCS);  
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_BACK);
}

void HOTOL::SetAttControlsIHOTOL()
{
	bAttSet = true;
	THRUSTER_HANDLE th_rot[2];

	th_rot[0] = CreateThruster (_V(0,-0.97, 5.25), _V(0, 1,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.97,-5.25), _V(0,-1,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-0.6,1.97, 23.65), _V(-0.707,-0.707,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 0.6,1.97, 23.65), _V( 0.707,-0.707,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.01,3.41,-17.21), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.01,3.41,-17.21), _V(0, 1.0,0.0),texRCS);  
	thg_pitchup=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHUP);

	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,1,0),MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,1,0),MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-0.6,1.97, 23.65), _V(-0.707,-0.707,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 0.6,1.97, 23.65), _V( 0.707,-0.707,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.04,2.72,-17.21), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.04,2.72,-17.21), _V(0,-1.0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_UP);

	th_rot[0] = CreateThruster (_V(0,-0.2,-5.25), _V(0,-1,0),MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,-0.75,5.25), _V(0,-1,0),MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-0.6,1.97, 23.65), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 0.6,1.97, 23.65), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.01,3.41,-17.21), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.01,3.41,-17.21), _V(0, 1.0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_DOWN);

	th_rot[0] = CreateThruster (_V(0,0, 5), _V(0,-1,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-5), _V(0, 1,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-0.6,1.97, 23.65), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 0.6,1.97, 23.65), _V(0, 1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.04,2.72,-17.21), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.04,2.72,-17.21), _V(0,-1.0,0.0),texRCS);  
	thg_pitchdown=CreateThrusterGroup (th_rot, 2, THGROUP_ATT_PITCHDOWN);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V(-1,0,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V( 1,0,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 0.6,1.97, 23.65), _V( 1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.24,3.22,-17.21), _V(-1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.24,2.94,-17.21), _V(-1.0,0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWLEFT);

	th_rot[0] = CreateThruster (_V(0,0, 4), _V( 1,0,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(0,0,-4), _V(-1,0,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-0.6,1.97, 23.65), _V(-1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.24,3.22,-17.21), _V( 1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.24,2.94,-17.21), _V( 1.0,0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_YAWRIGHT);

	th_rot[0] = CreateThruster (_V(2,0.0,0.0), _V(-1,0,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 0.6,1.97, 23.65), _V( 1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.24,3.22,-17.21), _V( 1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.24,2.94,-17.21), _V( 1.0,0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_LEFT);
	th_rot[0] = CreateThruster (_V(-2,0.0,0.0), _V( 1,0,0), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-0.6,1.97, 23.65), _V(-1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.24,3.22,-17.21), _V(-1.0,0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.24,2.94,-17.21), _V(-1.0,0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_RIGHT);

	th_rot[0] = CreateThruster (_V(13,0,0), _V(0,-1,0), MAX_ATT_FIRST/4, ph_Hotol, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, 1,0), MAX_ATT_FIRST/4, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.04,2.72,-17.21), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.01,3.41,-17.21), _V(0, 1.0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKRIGHT);

	th_rot[0] = CreateThruster (_V(13,0, 0), _V(0, 1,0), MAX_ATT_FIRST/4, ph_Hotol, ISP_FIRST_SL);
	th_rot[1] = CreateThruster (_V(-13,0,0), _V(0, -1,0), MAX_ATT_FIRST/4, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.04,2.72,-17.21), _V(0,-1.0,0.0),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.01,3.41,-17.21), _V(0, 1.0,0.0),texRCS);  
	CreateThrusterGroup (th_rot, 2, THGROUP_ATT_BANKLEFT);

	th_rot[0] = CreateThruster (_V(0,0, -4), _V(0,0,1), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-3.047, 3.08,-17.47), _V(0,0,-1),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 3.047, 3.08,-17.47), _V(0,0,-1),texRCS);  
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_FORWARD);
	th_rot[0] = CreateThruster (_V(0,0, 4), _V(0,0,-1), MAX_ATT_FIRST, ph_Hotol, ISP_FIRST_SL);
	AddExhaust (th_rot[0], 1.6, 0.5, _V( 0.11, 1.97, 23.77), _V(0,0,1),texRCS);  
	AddExhaust (th_rot[0], 1.6, 0.5, _V(-0.11, 1.97, 23.77), _V(0,0,1),texRCS);  
	CreateThrusterGroup (th_rot, 1, THGROUP_ATT_BACK);
}
