/**
*
* $Id: CVEL-HOTOL.h,v 0.1 2004/11/20 18:03:45 erv Exp $
* (c) 2004 Sputnik -- erik_h_anderson@hotmail.com
* Based on CVEL Centaur-G code
* Ervins Reinverts -- ervins@gmail.com
* Based on CVEL Fregat code
* (c) 2004 David Henderson -- orbiter@aibs.org.uk
* Licence LGPL -- http://www.gnu.org/licenses/lgpl.txt
*
* You may use this source code in your addon provided that if you
* publish your addon either privately or publicly the end user
* recieves a copy of the source code including any modifications
* you may have made. 
*
*/


#include "../../include/cvel.h"
#include "../../include/cvelmessage.h"
#include "../../include/orbitersdk.h"
#include "../../include/OrbiterSoundSDK3.h"
#include "../../include/guidance.h"
#include <cstdio>
#include <cmath>
#include <string>

#define InitFuelLevel 1

// All data from astronautix.com
const double ISP_FIRST_SL		=     425.0*G; 
const double ISP_FIRST_VAC		=     458.0*G; //***
const double ISP_SECOND_SL		=     425.0*G; 
const double ISP_SECOND_VAC		=     458.0*G; //***
const double ISP_IHOTOL_SL		=     352.0*G; 
const double ISP_IHOTOL_VAC		=     455.0*G; //***
const double THRUST_IHOTOL_VAC	=  200000.0*G; //***
const double ISP_RCS_VAC		=     425.0*G; //***
const double ISP_JET_VAC		=30000000.0*G;
const double ISP_BJET_VAC		=   30500.0*G;
const double THRUST_FIRST_VAC	=  315320.0*G/4.0;
//const double THRUST_FIRST_VAC	=   37500.0*G; //***
const double THRUST_JET_VAC		=   37500.0*G*3; //***
const double MAX_ATT_FIRST		=    1660.0*G;
const double JET_THRUST			=   17000.0*G;
const double BJET_THRUST		=   18000.0*G;
const double SECOND_PROP_MASS	=  200000.0;
//const double SECOND_PROP_MASS	=  280000.0;
const double IHOTOL_PROP_MASS	=  230000.0;
//const double RCS_PROP_MASS		=	 6000.0;
const double BJET_PROP_MASS		=   12000.0;
const double JET_PROP_MASS		=    2000.0;
const double HOTOL_DRYMASS		=   42000.0;
//const double HOTOL_DRYMASS		=   30000.0; //***
const double IHOTOL_DRYMASS		=   33100.0; //***
const double TANK_DRYMASS		=   16000.0; //***
const double BOOSTER_DRYMASS	=   97250.0; //***
const double BOOSTER_RCSMASS	=     800.0; //***

const VECTOR3 HOTOL_OFS			= _V(0,0,0);
const VECTOR3 TANK_OFS			= _V(0,0,0);
const VECTOR3 BOOSTER_OFS		= _V(0,0,0);
const VECTOR3 EJSEAT_OFS		= _V(-0.462,-0.924,11.77);
const VECTOR3 EJSEAT_OFS2		= _V( 0.462,-0.924,11.77);
const VECTOR3 ASTRO_OFS			= _V(-0.462,-0.792,11.99);
const VECTOR3 ASTRO_OFS2		= _V( 0.462,-0.792,11.99);
const VECTOR3 EVA_OFS			= _V( 2.86,0.6,7.2);
const VECTOR3 CAMERA_OFS		= _V( 0, 2.0,25.0);
const VECTOR3 ICAMERA_OFS		= _V( 0, 2.0,5.0);
const VECTOR3 OFS_SAT			= _V(0,0,0);

const double GEAR_OPERATING_SPEED = 0.25;
const double PLDOORS_OPERATING_SPEED = 0.07125;
const double BRAKES_OPERATING_SPEED = 0.25;
const double HATCH_OPERATING_SPEED = 0.125;
const double WINGS_OPERATING_SPEED = 0.25;
const double JETS_OPERATING_SPEED = 0.5;
const double NOZZLES_OPERATING_SPEED = 0.25;

const double MAX_GRAPPLING_DIST		= 2.0;

const double ROLL = 0.0;            // 180.0 to go to orbit heads-down.  Intermediate values probably won't work.

#define PLAYGEAR					 20		// the gear sound
#define PLAYBANG					 24		// the jettison sound
#define PLAYJETT					 25     // entry shield jettison sound
#define PLAYJET						 26     // the jet noise sound

/* make your own vessel class (named HOTOL here, for example)
   subclassed from COMMON_VESSEL_EX */

class HOTOL : public COMMON_VESSEL_EX, GUIDANCE {
public:
	HOTOL (OBJHANDLE hObj, int fmodel);
	~HOTOL();

	enum DoorStatus { DOOR_CLOSED, DOOR_OPEN, DOOR_CLOSING, DOOR_OPENING } gear_status, pldoors_status, brakes_status, hatch_status, wings_status, jets_status, nozzles_status, trolley_status;

	void MakeAnimations();
	void MakeAirfoils();
	void MakeControlSurfaces();
//	void BoosterSwitchToJets();
	void UpdateAnimations();
	void ActivateLandingGear (DoorStatus action);
	void RevertLandingGear();
	void ActivatePayloadBayDoors (DoorStatus action);
	void RevertPayloadBayDoors();
	void ActivateBrakes (DoorStatus action);
	void RevertBrakes();
	void ActivateNozzles (DoorStatus action);
	void RevertNozzles();
//	void SeparateEVA();
	void SetGearParameters (double state);
	void AttachSweep();
	void TimeStep(  double simt ); // need a timestep loop to conduct the GPDS polling
	void VectorThrust();
	void AirBreathingEngine();
	void VisualCreated( VESSEL *vessel, VISHANDLE vis, int refcount ); // must use VisualCreated() instead of ovcVisualCreated()
	void SetClassCaps(  FILEHANDLE cfg );
	void LoadState(  FILEHANDLE scn, void *vs );
	void SaveState(  FILEHANDLE scn );
	void ProgramAuto();
	void PlaceComponents();

	/* CVE-lite requirement ! */
	// Create a function that can be called as many times as you like. It is
	// vital to have a function like this that cleans out all the
	// thrusters and meshes and draws/recreates them again.
	// This is so that you can respond immediately to GPDS requests.
	// Here I've called it SetMainStage() it can be Refresh() or anything you like.
	void SetMainStage(); 
	void SetTrolleyStage();
	void SetHOTOLStage();  
	void SetIHOTOLStage();  
	void SetAttControlsHOTOL();    // this is to set up the thrusters. I like to keep them separate.
	void SetAttControlsIHOTOL();    // this is to set up the thrusters. I like to keep them separate.
	void JettisonPart();
	bool AzInput (void *id, char *str, void *data);
	bool IncInput (char *rstr);
	int  ConsumeKey(  const char *keystate );
	int  ConsumeBufferedKey(DWORD dwKey, bool fPressed, const char *keystate);
	double pitchinput, yawinput, rollinput;
	VESSELSTATUS vst;
	int stage;


// Overloaded callback functions
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkLoadStateEx (FILEHANDLE scn, void *vs);
	void clbkSaveState (FILEHANDLE scn);
	void clbkPostStep (double simt, double simdt, double mjd);
	int  clbkConsumeBufferedKey (DWORD key, bool down, char *kstate);
	void clbkVisualCreated (VISHANDLE vis, int refcount);
	void clbkVisualDestroyed (VISHANDLE vis, int refcount);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);

protected:
	double GetCPitch(double t);
	void Lateral_Guidance(VESSEL *vessel, double SimT, double SimT_Last);
	void Guidance_Routine(VESSEL *vessel, double SimT, double SimT_Last);
	void GetInclinationSP();
	void InitialiseGuidance();


	bool Guidance() { return guidance; };
	void Guidance( bool g ) { guidance = g; };

	void DisplayInclinationSetPoint( bool d ) { DisplayIncSP=d; };
	bool DisplayInclinationSetPoint() { return DisplayIncSP; };

	double initialgamma, initialheading, launchrotate;


	PROPELLANT_HANDLE ph_Hotol;
	THRUSTER_HANDLE   th_first[6];
	THGROUP_HANDLE    thg_first,thg_pitchup, thg_pitchdown;
	MESHHANDLE		  hHOTOL,hIHOTOL,hHOTOLTk,hSeat,hAstro,hSeat2,hAstro2,hDummy;
	ANIMATIONCOMPONENT_HANDLE Noz1, Noz2;
//	ANIMATIONCOMPONENT_HANDLE wing1, wing2, wing3, wing4, Gear1, Gear2, Gear3, Gear4, Gear5, Gear6, Gear7,
//		PLDr1, PLDr2, Hatch1, Jets1, Nozl1;
//	ANIMATIONCOMPONENT_HANDLE wing11, wing12, wing13, wing14, Gear11, Gear12, Gear13, Gear14, Gear15, Gear16, Gear17,
//		Gear18, Gear19, Jets11, Jets12, Jets13, Nozl11, Elev11, Rudd11, Elev12, Rudd12, Elev13, Rudd13, Elev14, Rudd14;
	SURFHANDLE		  tex,texRCS, retex;
	UINT anim_rudder,anim_laileron,anim_raileron,anim_elevator,anim_elevatortrim;
	UINT anim_gear;     // handle for landing gear animation
	UINT anim_pldoors;     // handle for payload door animation
	UINT anim_brakes;	// handle for speedbrake animation
	UINT anim_nozzles;	// handle for nozzles animation
	UINT anim_trolley, anim_trolley2;	// handle for trolley animation
	double gear_proc;
	double pldoors_proc;
	double brakes_proc;
	double nozzles_proc;
	double trolley_proc;
	bool BrakesOn, havesound, bShieldJettisoned, bDeorbitFired, bThrustTerminated, bEjected, bEVA, 
		bAbort, AbortChecked, bAttSet,bBoosterMade, bAutoProgrammed, bControlsMade;
	UINT Exh1,Exh2,Exh3,Exh4;
	PSTREAM_HANDLE stream_a_1,stream_b_1,stream_a_2,stream_b_2,stream_a_3,stream_b_3,stream_a_4,stream_b_4;
	VECTOR3 Vold, Vnew;
	double dt, oldsimt;
	int MyID;
	ATTACHMENTHANDLE Baypos, sat_attach, Nose_atch1, Nose_atch2;
	VECTOR3 gpos, grms, pos, dir, rot;
	OBJHANDLE hV, BayVessel, hvessel;
	VESSEL *BayV;
	DWORD i, j;
	double atchmass;
	DOCKHANDLE hDock, Dock0, Dock1, Dock2;
	bool Firstsweep;
	int numengines;
	double tempP, tempY, tempR;
	VECTOR3 AngularVel;
	bool autopilot;
	bool engineignited, bgoingsouth;
	double last_simt;
	double ignition_time;
	double density;
	double displaytime, simt, d_heading;
	bool first_guidance_pass, AnimationsMade;
	double MET, diff, target, hdg, diverge, yawangle, targetrate, rate, amount;
	AIRFOILHANDLE BstrWing, BstrRudder;
	char tvmesh[80];
	bool ihotol;

};








/* basic vessel class setup */

void HOTOL::clbkSetClassCaps (FILEHANDLE cfg)
{
	SetClassCaps(cfg);
}

void HOTOL::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
	LoadState(  scn, vs );
}

void HOTOL::clbkSaveState (FILEHANDLE scn)
{
	SaveState(  scn );
}

void HOTOL::clbkPostStep (double simt, double simdt, double mjd)
{
	TimeStep(simt);
}

void HOTOL::clbkVisualCreated (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

void HOTOL::clbkVisualDestroyed (VISHANDLE _vis, int refcount)
{
	SetMainStage( );
}

int HOTOL::clbkConsumeBufferedKey (DWORD dwKey, bool fPressed, char *keystate)
{
	int ret = ConsumeBufferedKey( dwKey, fPressed, keystate );
	return ret;
}

bool HOTOL::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	// show gear deployment status
	if (gear_status == DOOR_OPEN || (gear_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5)) {
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
			skp->Rectangle (cx-d/2, cy-d*5, cx+d/2, cy-d*4);
			skp->Rectangle (cx-d*3, cy-d*2, cx-d*2, cy-d);
			skp->Rectangle (cx+d*2, cy-d*2, cx+d*3, cy-d);
		}
	}

	// show pldoors deployment status
	if (pldoors_status == DOOR_OPEN || (pldoors_status >= DOOR_CLOSING && fmod (oapiGetSimTime(), 1.0) < 0.5)) {
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
//			skp->Arc (cx, cy-d*8.5, cx+d*4, cy-d*6.5, cx, cy-d*6, cx+d*4, cy-d*7.5);
//			skp->Arc (cx-d*4, cy-d*8.5, cx, cy-d*6.5, cx-d*4, cy-d*7.5, cx, cy-d*6);
			skp->Line(cx-d*4, cy-d*8.5, cx-d*3, cy-d*7.5);
			skp->LineTo(cx-d*2, cy-d*7.0);
			skp->LineTo(cx-d, cy-d*6.65);
			skp->LineTo(cx, cy-d*6.5);
			skp->LineTo(cx+d, cy-d*6.65);
			skp->LineTo(cx+d*2, cy-d*7.0);
			skp->LineTo(cx+d*3, cy-d*7.5);
			skp->LineTo(cx+d*4, cy-d*8.5);
			skp->Text (cx-d, cy-d*8, "BAY", 3);
		}
	}


	// show brakes deployment status
	if (BrakesOn || brakes_proc > 0) {
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		if((((!BrakesOn && brakes_proc > 0) || (BrakesOn && brakes_proc < 1.0)) && 
				fmod (oapiGetSimTime(), 0.5) < 0.25)||(BrakesOn && brakes_proc == 1.0))
		{
			//Flash indicator
			if (cx >= -d*3 && cx < hps->W+d*3 && cy >= d && cy < hps->H+d*5) {
				skp->Text (cx-d*9, cy+d*1.2, "BRAKES", 6);
			}
		}
	}
	//Draw Equivalent airspeed & pressure altitude
	if(GetAtmPressure() > 0.0)
	{
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		char pszBuffer[40];
		int bx = 0.5 * cx;
		const double EAS_EXP = 2.0/7.0;
		const double P0 = 1.01325E6;

		double fEAS = 661.48 * sqrt(5.0 * GetAtmPressure()/ATMP * (pow(GetDynPressure() / GetAtmPressure() + 1, EAS_EXP) - 1.0));

		sprintf(pszBuffer, "%6.1f KEAS", fEAS);

		skp->Text(bx, cy - d, pszBuffer, 11);

		bx = 0.5 * (cx + hps->W) - d * 8;

		double fHp = (1.0 - pow(GetAtmPressure()/ATMP, 0.190263)) * (288.15/0.00198122);

		long H_press = (long)(fHp);
		H_press = H_press - H_press % 50;

		sprintf(pszBuffer, "%6d'", H_press);

		skp->Text(bx, cy - d, pszBuffer, 7);
	}

	//Draw Radar altitude
	if(GetAltitude() < 2500.0 * 0.3048)
	{
		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;
		char pszBuffer[40];
		int bx = 0.5 * (cx + hps->W) - d * 8;
		const double GEAR_Y = -1.35;

		sprintf(pszBuffer, "%6dR", static_cast<long>((GetAltitude() + GEAR_Y)/0.3048));

		skp->Text(bx, cy + d, pszBuffer, 7);
	}


	//Draw G-Meter

		int d = hps->Markersize/2;
		int cx = hps->CX, cy = hps->CY;

		char pszBuffer[40];
		int bx = 0.5 * cx;

		VECTOR3 ForceVec, WeightVec;
		GetForceVector(ForceVec);
		GetWeightVector(WeightVec);
		ForceVec -= WeightVec;
		double G_level = sqrt(pow(ForceVec.x,2)+pow(ForceVec.y,2)+pow(ForceVec.z,2)) / (9.81*GetMass());

		sprintf(pszBuffer, "  %.2f G", G_level);

		skp->Text(bx, cy + d, pszBuffer, strlen(pszBuffer));


	return true;
}

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new HOTOL (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (HOTOL*)vessel;
}

